# ------------------------------------------------------------------------------
# Test Calendar Pages
# ------------------------------------------------------------------------------
import sys
import datetime as dt
from unittest.mock import Mock
from django_bs_test import TestCase
from django.contrib.auth.models import User
from django.test import RequestFactory
from django.utils import translation
from django.urls import reverse
from wagtail.admin.edit_handlers import get_form_for_model
from wagtail.core.models import Site, Page
from ls.joyous.models.calendar import (CalendarPage, SpecificCalendarPage,
        CalendarPageForm, GeneralCalendarPage)
from ls.joyous.models.events import SimpleEventPage
from ls.joyous.models.groups import get_group_model
from .testutils import freeze_timetz, getPage

GroupPage = get_group_model()

# ------------------------------------------------------------------------------
class TestCalendar(TestCase):
    def setUp(self):
        self.user = User.objects.create_user('i', 'i@j.test', 's3(r3t')
        calendar = CalendarPage(owner  = self.user,
                                slug  = "events",
                                title = "Events",
                                default_view = "L")
        Page.objects.get(slug='home').add_child(instance=calendar)
        calendar.save_revision().publish()
        event = SimpleEventPage(owner = self.user,
                                slug  = "tree-planting",
                                title = "Tree Planting",
                                date      = dt.date(2011,6,5),
                                time_from = dt.time(9,30),
                                time_to   = dt.time(11,0))
        calendar.add_child(instance=event)
        event.save_revision().publish()

    def testMonthView(self):
        response = self.client.get("/events/2012/03/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(select('thead a')), 4)
        self.assertEqual(len(select(".joy-cal__weekday--sun")), 1)
        month = select("tr.joy-cal__headings th.joy-cal__heading .joy-cal__month-name")[0]
        self.assertEqual(month.string.strip(), "March")
        self.assertEqual(len(select("tbody tr")), 5)
        self.assertEqual(len(select("tbody td")), 35)
        self.assertEqual(len(select("tbody td.joy-cal__day")), 31)
        self.assertEqual(len(select("tbody td.joy-cal__no-day")), 4)
        holidayNames = [holiday.string.strip() for holiday in
                        select("tbody td.joy-cal__day .joy-cal__holiday-name")]
        self.assertEqual(holidayNames,
                         ["Taranaki Anniversary Day", "Otago Anniversary Day"])

    def testMonthAbbr(self):
        response = self.client.get("/events/2012/Apr/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(select(".joy-cal__weekday--sun")), 1)
        month = select(".joy-cal__month-name")[0]
        self.assertEqual(month.string.strip(), "April")

    def testWeekView(self):
        response = self.client.get("/events/2012/W11/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(select('thead a')), 4)
        self.assertEqual(len(select(".joy-cal__weekday--sun")), 1)
        self.assertEqual(len(select("tbody tr")), 1)
        self.assertEqual(len(select(".joy-cal__day")), 7)
        holidayDates = select(".joy-cal__date--holiday")
        self.assertEqual(len(holidayDates), 1)
        self.assertEqual(holidayDates[0].string.strip(), "12 Mar")
        holidayNames = select(".joy-cal__holiday-name")
        self.assertEqual(holidayNames[0].string.strip(),
                         "Taranaki Anniversary Day")

    def testDayWithOneEvent(self):
        response = self.client.get("/events/2011/6/5/")
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, "/events/tree-planting/")

    def testDayWithEvents(self):
        calendar = getPage("/home/events/")
        event1 = SimpleEventPage(owner = self.user,
                                 slug  = "memory",
                                 title = "We remember",
                                 date      = dt.date(2011,6,8),
                                 time_from = dt.time(9,30),
                                 time_to   = dt.time(10,30))
        calendar.add_child(instance=event1)
        event1.save_revision().publish()
        event2 = SimpleEventPage(owner = self.user,
                                slug  = "action",
                                title = "Make things better",
                                date      = dt.date(2011,6,8),
                                time_from = dt.time(11,30),
                                time_to   = dt.time(16,0))
        calendar.add_child(instance=event2)
        event2.save_revision().publish()
        response = self.client.get("/events/2011/6/8/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(select(".joy-cal-list")), 1)
        events = select(".joy-ev-item")
        self.assertEqual(len(events), 2)
        title = events[0].select("a.joy-title__link")[0]
        self.assertEqual(title.string.strip(), "We remember")
        self.assertEqual(title['href'], "/events/memory/")

    def testUpcomingEvents(self):
        response = self.client.get("/events/upcoming/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(select(".joy-cal-list--upcoming")), 1)
        self.assertEqual(len(select(".joy-cal-list--upcoming .joy-ev-item")), 0)

    def testUpcomingEventsInvalidPage(self):
        response = self.client.get("/events/upcoming/?page=99")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(select(".joy-cal-list--upcoming")), 1)
        self.assertEqual(len(select(".joy-cal-list--upcoming .joy-ev-item")), 0)

    def testPastEvents(self):
        response = self.client.get("/events/past/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(select(".joy-cal-list--past")), 1)
        events = select(".joy-cal-list--past .joy-ev-item")
        self.assertEqual(len(events), 1)
        title = events[0].select("a.joy-title__link")[0]
        self.assertEqual(title.string.strip(), "Tree Planting")
        self.assertEqual(title['href'], "/events/tree-planting/")
        when = events[0].select(".joy-ev-when")[0]
        self.assertEqual(when.string.strip(),
                         "Sunday 5th of June 2011 at 9:30am to 11am")

    def testPastEventsInvalidPage(self):
        response = self.client.get("/events/past/?page=99")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(select(".joy-cal-list--past")), 1)
        self.assertEqual(len(select(".joy-cal-list--past .joy-ev-item")), 1)

    def testRouteDefault(self):
        response = self.client.get("/events/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(select(".joy-cal-list--upcoming")), 1)
        response = self.client.get("/events/?view=weekly")
        self.assertEqual(response.status_code, 200)
        select = response.soup.select
        self.assertEqual(len(select(".joy-cal__week-name")), 1)
        response = self.client.get("/events/?view=monthly")
        select = response.soup.select
        self.assertEqual(len(select(".joy-cal__month-name")), 1)

    def testMiniMonthView(self):
        response = self.client.get("/events/mini/2011/06/")
        self.assertEqual(response.status_code, 404)

    def testMiniMonthAjaxView(self):
        response = self.client.get("/events/mini/2011/06/",
                                   HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(select("table.joy-minical thead tr .joy-minical__weekday--sun")), 1)
        month = select("tr th .joy-minical__month-name")[0]
        self.assertEqual(month.string.strip(), "June")
        self.assertEqual(len(select("tbody tr")), 5)
        self.assertEqual(len(select("tbody td")), 35)
        self.assertEqual(len(select("tbody td.joy-minical__day")), 30)
        self.assertEqual(len(select("tbody td.joy-minical__no-day")), 5)
        event = select("tbody td.joy-minical__day a.joy-minical__date--event-link")[0]
        self.assertEqual(event.string.strip(), "5")
        self.assertEqual(event['href'], "/events/2011/06/05/")
        self.assertEqual(event['title'], "Tree Planting")

    def testInvalidDates(self):
        invalidDates = ["2012/13", "2008/W54", "2099/W53"]
        for date in invalidDates:
            with self.subTest(date=date):
                response = self.client.get("/events/{}/".format(date))
                self.assertEqual(response.status_code, 404)

    def testCalendarStartMonth(self):
        response = self.client.get("/events/1900/1/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        links = select('thead a')
        self.assertEqual(len(links), 2)
        self.assertEqual(links[0].get('title'), "Next month")
        self.assertEqual(links[0].get('href'), "/events/1900/2/")
        self.assertEqual(links[1].get('title'), "Next year")
        self.assertEqual(links[1].get('href'), "/events/1901/1/")

    def testCalendarFinishMonth(self):
        response = self.client.get("/events/2099/12/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        links = select('thead a')
        self.assertEqual(len(links), 2)
        self.assertEqual(links[0].get('title'), "Previous month")
        self.assertEqual(links[0].get('href'), "/events/2099/11/")
        self.assertEqual(links[1].get('title'), "Previous year")
        self.assertEqual(links[1].get('href'), "/events/2098/12/")

    def testCalendarStartWeek(self):
        response = self.client.get("/events/1900/W1/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        links0 = select('.joy-view-choices a')
        self.assertEqual(len(links0), 3)
        self.assertEqual(links0[0].get_text(), "List View")
        self.assertEqual(links0[1].get_text(), "This Week")
        self.assertEqual(links0[2].get_text(), "Monthly View")
        self.assertEqual(links0[2].get('href'), "/events/1900/1/")
        links1 = select('thead a')
        self.assertEqual(len(links1), 2)
        self.assertEqual(links1[0].get('title'), "Next week")
        self.assertEqual(links1[0].get('href'), "/events/1900/W2/")
        self.assertEqual(links1[1].get('title'), "Next year")
        self.assertEqual(links1[1].get('href'), "/events/1901/W1/")

    def testCalendarFinishWeek(self):
        response = self.client.get("/events/2099/W52/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        links0 = select('.joy-view-choices a')
        self.assertEqual(len(links0), 3)
        self.assertEqual(links0[0].get_text(), "List View")
        self.assertEqual(links0[1].get_text(), "This Week")
        self.assertEqual(links0[2].get_text(), "Monthly View")
        self.assertEqual(links0[2].get('href'), "/events/2099/12/")
        links1 = select('thead a')
        self.assertEqual(len(links1), 2)
        self.assertEqual(links1[0].get('title'), "Previous week")
        self.assertEqual(links1[0].get('href'), "/events/2099/W51/")
        self.assertEqual(links1[1].get('title'), "Previous year")
        self.assertEqual(links1[1].get('href'), "/events/2098/W52/")

    def testNextYearW53(self):
        response = self.client.get("/events/2098/W53/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        links = select('thead a')
        self.assertEqual(len(links), 3)
        self.assertEqual(links[0].get('title'), "Previous week")
        self.assertEqual(links[0].get('href'), "/events/2098/W52/")
        self.assertEqual(links[1].get('title'), "Next week")
        self.assertEqual(links[1].get('href'), "/events/2099/W1/")
        self.assertEqual(links[2].get('title'), "Previous year")
        self.assertEqual(links[2].get('href'), "/events/2097/W52/")

    def testFutureDates(self):
        response = self.client.get("/events/2525/1/")
        self.assertEqual(response.status_code, 404)
        response = self.client.get("/events/2100/W1/")
        self.assertEqual(response.status_code, 404)

# ------------------------------------------------------------------------------
class TestCalendarPageForm(TestCase):
    def setUp(self):
        self.user = User.objects.create_superuser('i', 'i@j.test', 's3(r3t')
        self.request = RequestFactory().get("/test")
        self.request.user = self.user
        self.request.session = {}
        self.home = Page.objects.get(slug='home')
        self.page = CalendarPage(owner  = self.user,
                                 slug  = "events",
                                 title = "Events")
        self.home.add_child(instance=self.page)
        self.page.save_revision().publish()

    def testImportPanel(self):
        CalendarPageForm.registerImportHandler(Mock())
        panel = CalendarPage.settings_panels[-1]
        self.assertFalse(panel._show())
        panel.instance = self.page
        panel.request  = self.request
        self.assertFalse(panel._show())
        setattr(self.page, '__joyous_edit_request', self.request)
        self.assertTrue(panel._show())
        delattr(self.page, '__joyous_edit_request')

    def testSave(self):
        Form = get_form_for_model(CalendarPage, form_class=CalendarPageForm)
        setattr(self.page, '__joyous_edit_request', self.request)
        form = Form(instance=self.page, parent_page=self.home)
        handler = Mock()
        CalendarPageForm.registerImportHandler(handler)
        form.cleaned_data = {'utc2local': True,
                             'upload':    "FILE"}
        form.save()
        handler.load.assert_called_with(self.page, self.request,
                                        "FILE", utc2local=True)


# ------------------------------------------------------------------------------
class TestFrançais(TestCase):
    def setUp(self):
        translation.activate('fr')
        self.user = User.objects.create_superuser('i', 'i@j.test', 's3(r3t')
        calendar = CalendarPage(owner  = self.user,
                                slug  = "calendrier",
                                title = "Calendrier")
        Page.objects.get(slug='home').add_child(instance=calendar)
        calendar.save_revision().publish()
        event = SimpleEventPage(owner = self.user,
                                slug  = "plantation-d-arbres",
                                title = "Plantation d'arbres",
                                date      = dt.date(2011,6,5),
                                time_from = dt.time(9,30),
                                time_to   = dt.time(11,0))
        calendar.add_child(instance=event)
        event.save_revision().publish()

    def tearDown(self):
        translation.deactivate()

    def testMonthView(self):
        response = self.client.get("/calendrier/2012/03/")
        select = response.soup.select
        self.assertEqual(response.status_code, 200)
        month = select(".joy-cal__month-name")[0]
        self.assertEqual(month.string.strip(), "Mars")

    def testWeekView(self):
        response = self.client.get("/calendrier/2012/W11/")
        select = response.soup.select
        holidayDate = select(".joy-cal__date--holiday")
        self.assertEqual(holidayDate[0].string.strip(), "12 Mar")
        holidayName = select(".joy-cal__holiday-name")
        self.assertEqual(holidayName[0].string.strip(),
                         "Taranaki Anniversary Day")

# ------------------------------------------------------------------------------
class TestSpecificCalendar(TestCase):
    def setUp(self):
        SpecificCalendarPage.is_creatable = True
        self.user = User.objects.create_user('i', 'i@j.test', 's3(r3t')
        self.request = RequestFactory().get("/test")
        self.request.user = self.user
        self.request.session = {}
        self.calendar1 = SpecificCalendarPage(owner  = self.user,
                                              slug  = "calendar1",
                                              title = "Red Team Calendar")
        home = getPage("/home/")
        home.add_child(instance=self.calendar1)
        self.calendar1.save_revision().publish()
        event = SimpleEventPage(owner = self.user,
                                slug  = "football",
                                title = "Football Game",
                                uid   = "570ed9c4-4503-4b45-b15e-c99faed9c531",
                                date      = dt.date(2011,6,5),
                                time_from = dt.time(9,30),
                                time_to   = dt.time(11,0))
        self.calendar1.add_child(instance=event)
        event.save_revision().publish()
        self.calendar2 = SpecificCalendarPage(owner  = self.user,
                                              slug  = "calendar2",
                                              title = "Green Team Calendar")
        home.add_child(instance=self.calendar2)
        self.calendar2.save_revision().publish()
        event = SimpleEventPage(owner = self.user,
                                slug  = "tree-planting",
                                title = "Tree Planting",
                                uid   = "eb50e787-12bf-477b-8493-c4414ac001ca",
                                date      = dt.date(2011,6,5),
                                time_from = dt.time(9,30))
        self.calendar2.add_child(instance=event)
        event.save_revision().publish()

    def testEventsOnDay(self):
        evod = self.calendar1._getEventsOnDay(self.request, dt.date(2011,6,5))
        self.assertEqual(evod.date, dt.date(2011,6,5))
        self.assertEqual(len(evod.all_events), 1)
        self.assertEqual(len(evod.days_events), 1)
        event = evod.days_events[0]
        self.assertEqual(event.title, "Football Game")

    def testEventsByDay(self):
        evods = self.calendar1._getEventsByDay(self.request, dt.date(2011,7,1),
                                               dt.date(2011,7,10))
        self.assertEqual(len(evods), 10)
        evod = evods[0]
        self.assertEqual(evod.date, dt.date(2011,7,1))
        self.assertEqual(len(evod.all_events), 0)

    def testEventsByWeek(self):
        weeks = self.calendar1._getEventsByWeek(self.request, 2011, 6)
        self.assertEqual(len(weeks), 5)
        self.assertIsNone(weeks[0][0])
        self.assertIsNone(weeks[0][2])
        evod = weeks[1][0]
        self.assertEqual(evod.date, dt.date(2011,6,5))
        self.assertEqual(len(evod.all_events), 1)
        event = evod.days_events[0]
        self.assertEqual(event.title, "Football Game")
        self.assertEqual(event.page.slug, "football")

    @freeze_timetz("2011-01-21 15:00")
    def testUpcomingEvents(self):
        events = self.calendar2._getUpcomingEvents(self.request)
        self.assertEqual(len(events), 1)
        title, event, url = events[0]
        self.assertEqual(title, "Tree Planting")
        self.assertEqual(event.slug, "tree-planting")

    @freeze_timetz("2011-01-21 15:00")
    def testPastEvents(self):
        events = self.calendar2._getPastEvents(self.request)
        self.assertEqual(len(events), 0)

    def testGetEventFromUid(self):
        event = self.calendar1._getEventFromUid(self.request,
                                                "570ed9c4-4503-4b45-b15e-c99faed9c531")
        self.assertEqual(event.title, "Football Game")
        event = self.calendar1._getEventFromUid(self.request,
                                                "eb50e787-12bf-477b-8493-c4414ac001ca")
        self.assertIsNone(event)

    def testGetAllEventsByDay(self):
        events = self.calendar2._getAllEvents(self.request)
        self.assertEqual(len(events), 1)
        self.assertEqual(events[0].title, "Tree Planting")

# ------------------------------------------------------------------------------
class TestGeneralCalendar(TestCase):
    def setUp(self):
        GeneralCalendarPage.is_creatable = True
        self.user = User.objects.create_user('i', 'i@j.test', 's3(r3t')
        self.request = RequestFactory().get("/test")
        self.request.user = self.user
        self.request.session = {}
        self.calendar = GeneralCalendarPage(owner  = self.user,
                                             slug  = "calendar",
                                             title = "My Calendar")
        home = getPage("/home/")
        home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()
        event = SimpleEventPage(owner = self.user,
                                slug  = "bbq",
                                title = "BBQ",
                                uid   = "a94c3211-08e5-4e36-9448-86a869a47d89",
                                date      = dt.date(2011,8,20),
                                time_from = dt.time(18,30))
        self.calendar.add_child(instance=event)
        event.save_revision().publish()
        self.group = GroupPage(slug = "initech", title = "Initech Corporation")
        home.add_child(instance=self.group)
        self.group.save_revision().publish()
        event = SimpleEventPage(owner = self.user,
                                slug  = "planning-meeting",
                                title = "Planning to Plan",
                                uid   = "a96484da-e350-45c7-af03-97ca2bb173d6",
                                date      = dt.date(2011,8,20),
                                time_from = dt.time(8,0))
        self.group.add_child(instance=event)
        event.save_revision().publish()

    def testEventsOnDay(self):
        evod = self.calendar._getEventsOnDay(self.request, dt.date(2011,8,20))
        self.assertEqual(evod.date, dt.date(2011,8,20))
        self.assertEqual(len(evod.all_events), 2)
        self.assertEqual(len(evod.days_events), 2)
        events = evod.days_events
        self.assertEqual(events[0].title, "Planning to Plan")
        self.assertEqual(events[1].title, "BBQ")

    def testEventsByDay(self):
        evods = self.calendar._getEventsByDay(self.request, dt.date(2011,8,1),
                                              dt.date(2011,8,20))
        self.assertEqual(len(evods), 20)
        evod = evods[19]
        self.assertEqual(evod.date, dt.date(2011,8,20))
        self.assertEqual(len(evod.all_events), 2)
        events = evod.days_events
        self.assertEqual(events[0].title, "Planning to Plan")
        self.assertEqual(events[1].title, "BBQ")

    def testEventsByWeek(self):
        weeks = self.calendar._getEventsByWeek(self.request, 2011, 8)
        self.assertEqual(len(weeks), 5)
        self.assertIsNone(weeks[0][0])
        evod = weeks[2][6]
        self.assertEqual(evod.date, dt.date(2011,8,20))
        self.assertEqual(len(evod.all_events), 2)
        events = evod.days_events
        self.assertEqual(events[0].title, "Planning to Plan")
        self.assertEqual(events[1].title, "BBQ")

    @freeze_timetz("2011-08-21 15:00")
    def testUpcomingEvents(self):
        events = self.calendar._getUpcomingEvents(self.request)
        self.assertEqual(len(events), 0)

    @freeze_timetz("2011-08-21 15:00")
    def testPastEvents(self):
        events = self.calendar._getPastEvents(self.request)
        self.assertEqual(len(events), 2)
        title, event, url = events[0]
        self.assertEqual(title, "BBQ")
        self.assertEqual(event.slug, "bbq")
        title, event, url = events[1]
        self.assertEqual(event.slug, "planning-meeting")

    def testGetEventFromUid(self):
        event = self.calendar._getEventFromUid(self.request,
                                                "a94c3211-08e5-4e36-9448-86a869a47d89")
        self.assertEqual(event.title, "BBQ")
        event = self.calendar._getEventFromUid(self.request,
                                                "a96484da-e350-45c7-af03-97ca2bb173d6")
        self.assertEqual(event.title, "Planning to Plan")

    def testGetAllEventsByDay(self):
        events = self.calendar._getAllEvents(self.request)
        self.assertEqual(len(events), 2)
        self.assertEqual(events[0].title, "Planning to Plan")
        self.assertEqual(events[1].title, "BBQ")

# ------------------------------------------------------------------------------
class TestMultiCalendarCreate(TestCase):
    def setUp(self):
        self.user = User.objects.create_user('i', 'i@j.test', 's3(r3t')
        self.main = getPage("/home/")
        self.sub = Page(slug="nova", title="Nova Homepage")
        self.main.add_child(instance=self.sub)
        self.sub.save_revision().publish()
        Site.objects.create(hostname='nova.joy.test',
                            root_page_id=self.sub.id,
                            is_default_site=False)
        SpecificCalendarPage.is_creatable = True
        GeneralCalendarPage.is_creatable = True

    def testMainSiteAnotherCalendar(self):
        calendar = CalendarPage(owner  = self.user,
                                slug  = "events",
                                title = "Events",)
        self.main.add_child(instance=calendar)
        calendar.save_revision().publish()
        self.assertFalse(CalendarPage.can_create_at(self.main))

    def testSubSiteAnotherCalendar(self):
        calendar = CalendarPage(owner  = self.user,
                                slug  = "events",
                                title = "Events")
        self.main.add_child(instance=calendar)
        calendar.save_revision().publish()
        self.assertTrue(CalendarPage.can_create_at(self.sub))

    def testNoSiteAnotherCalendar(self):
        rogue = Page(slug="rogue", title="Rogue")
        self.assertFalse(CalendarPage.can_create_at(rogue))

    def testMainSiteAnotherSpecificCalendar(self):
        calendar = SpecificCalendarPage(owner  = self.user,
                                        slug  = "events",
                                        title = "Events")
        self.main.add_child(instance=calendar)
        calendar.save_revision().publish()
        self.assertTrue(SpecificCalendarPage.can_create_at(self.main))

    def testMainSiteAnotherGeneralCalendar(self):
        calendar = GeneralCalendarPage(owner  = self.user,
                                       slug  = "events",
                                       title = "Events")
        self.main.add_child(instance=calendar)
        calendar.save_revision().publish()
        self.assertFalse(GeneralCalendarPage.can_create_at(self.main))

    def testSubSiteAnotherGeneralCalendar(self):
        calendar = GeneralCalendarPage(owner  = self.user,
                                       slug  = "events",
                                       title = "Events")
        self.main.add_child(instance=calendar)
        calendar.save_revision().publish()
        self.assertFalse(GeneralCalendarPage.can_create_at(self.sub))

    def testCalendarMixture(self):
        general = GeneralCalendarPage(owner  = self.user,
                                      slug  = "events1",
                                      title = "Events")
        self.main.add_child(instance=general)
        general.save_revision().publish()
        self.assertTrue(CalendarPage.can_create_at(self.main))
        calendar = CalendarPage(owner  = self.user,
                                slug  = "events2",
                                title = "Events")
        self.main.add_child(instance=calendar)
        calendar.save_revision().publish()
        self.assertTrue(SpecificCalendarPage.can_create_at(self.main))
        specific = SpecificCalendarPage(owner  = self.user,
                                        slug  = "events3",
                                        title = "Events")
        self.main.add_child(instance=specific)
        specific.save_revision().publish()


# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Test Multiday Recurring Event Page
# ------------------------------------------------------------------------------
import sys
import datetime as dt
import pytz
import calendar
from django.test import TestCase, RequestFactory
from django.contrib.auth.models import User
from django.utils import timezone
from wagtail.core.models import Page, PageViewRestriction
from ls.joyous.utils.recurrence import Recurrence
from ls.joyous.utils.recurrence import DAILY, WEEKLY, YEARLY
from ls.joyous.utils.recurrence import SA, MO, TU, TH, FR
from ls.joyous.models.calendar import CalendarPage
from ls.joyous.models.events import MultidayRecurringEventPage, ExtraInfoPage
from .testutils import datetimetz, freeze_timetz

# ------------------------------------------------------------------------------
class Test(TestCase):
    def setUp(self):
        self.user = User.objects.create_user('i', 'i@joy.test', 's3cr3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        Page.objects.get(slug='home').add_child(instance=self.calendar)
        self.calendar.save()
        self.calendar.save_revision().publish()
        self.event = MultidayRecurringEventPage(
                               owner = self.user,
                               slug  = "team-retreat",
                               title = "Team Retreat",
                               repeat = Recurrence(dtstart=dt.date(2000,1,1),
                                                   freq=YEARLY,
                                                   bymonth=8,
                                                   byweekday=FR(1)),
                               num_days  = 3,
                               time_from = dt.time(18),
                               time_to   = dt.time(16,30))
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()

    def testGetEventsByDay(self):
        events = MultidayRecurringEventPage.events.byDay(dt.date(2017,8,1),
                                                         dt.date(2017,8,31))
        self.assertEqual(len(events), 31)
        evod = events[3]
        self.assertEqual(evod.date, dt.date(2017,8,4))
        self.assertEqual(len(evod.days_events), 1)
        self.assertEqual(len(evod.continuing_events), 0)
        evod = events[4]
        self.assertEqual(evod.date, dt.date(2017,8,5))
        self.assertEqual(len(evod.days_events), 0)
        self.assertEqual(len(evod.continuing_events), 1)
        evod = events[5]
        self.assertEqual(evod.date, dt.date(2017,8,6))
        self.assertEqual(len(evod.days_events), 0)
        self.assertEqual(len(evod.continuing_events), 1)

    def testStatus(self):
        with freeze_timetz("2014-08-01 17:00:00"):
            self.assertEqual(self.event.status_text, "")
        with freeze_timetz("2014-08-02 13:00:00"):
            self.assertEqual(self.event.status_text, "This event has started.")
        with freeze_timetz("2014-08-03 15:00:00"):
            self.assertEqual(self.event.status_text, "This event has started.")
        with freeze_timetz("2014-08-03 17:00:00"):
            self.assertEqual(self.event.status_text, "")

    def testNextOn(self):
        request = RequestFactory().get("/test")
        request.user = self.user
        request.session = {}
        oldEvent = MultidayRecurringEventPage(
                               owner = self.user,
                               slug  = "same-old-thing",
                               title = "Same Ol'",
                               repeat = Recurrence(dtstart=dt.date(1971,1,1),
                                                   until=dt.date(1982,1,1),
                                                   freq=WEEKLY,
                                                   byweekday=SA(1)),
                               num_days  = 2)
        self.calendar.add_child(instance=oldEvent)
        oldEvent.save_revision().publish()
        with freeze_timetz("1974-08-01 17:00:00"):
            self.assertEqual(oldEvent.next_date, dt.date(1974, 8, 3))
            self.assertEqual(oldEvent._nextOn(request), "Saturday 3rd of August ")
        with freeze_timetz("1982-01-01 17:00:00"):
            self.assertIsNone(oldEvent.next_date)
            self.assertEqual(oldEvent._nextOn(request), None)

    def testWhen(self):
        self.assertEqual(self.event.when,
                         "The first Friday of August for 3 days "
                         "starting at 6pm finishing at 4:30pm")

    def testAt(self):
        self.assertEqual(self.event.at.strip(), "6pm")

    @freeze_timetz("2035-04-03 10:00:00")
    def testPrevDate(self):
        self.assertEqual(self.event.prev_date, dt.date(2034, 8, 4))

    @freeze_timetz("2018-04-03 10:00:00")
    def testFutureExceptions(self):
        request = RequestFactory().get("/test")
        request.user = self.user
        request.session = {}
        info2018 = ExtraInfoPage(owner = self.user,
                                 overrides = self.event,
                                 except_date = dt.date(2018, 8, 3),
                                 extra_title = "Team Retreat 2018",
                                 extra_information = "Weekend at Bernie's")
        self.event.add_child(instance=info2018)
        exceptions = self.event._futureExceptions(request)
        self.assertEqual(len(exceptions), 1)
        info = exceptions[0]
        self.assertEqual(info.slug, "2018-08-03-extra-info")
        self.assertEqual(info.extra_title, "Team Retreat 2018")

    @freeze_timetz("2018-08-04 02:00:00")
    def testPastExcludeExtraInfo(self):
        info2018 = ExtraInfoPage(owner = self.user,
                                 overrides = self.event,
                                 except_date = dt.date(2018, 8, 3),
                                 extra_title = "Team Retreat 2018",
                                 extra_information = "Weekend at Bernie's")
        self.event.add_child(instance=info2018)
        before = self.event._past_datetime_from
        self.assertEqual(before, datetimetz(2017, 8, 4, 18))

# ------------------------------------------------------------------------------
class Test40Day(TestCase):
    def setUp(self):
        self.user = User.objects.create_user('i', 'i@joy.test', 's3cr3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        Page.objects.get(slug='home').add_child(instance=self.calendar)
        self.calendar.save()
        self.calendar.save_revision().publish()
        self.event = MultidayRecurringEventPage(
                               owner = self.user,
                               slug  = "ice-festival",
                               title = "Ice Festival",
                               repeat = Recurrence(dtstart=dt.date(2000,12,25),
                                                   until=dt.date(2020,1,31),
                                                   freq=YEARLY,
                                                   bymonth=12,
                                                   byweekday=MO(4)),
                               num_days  = 40)
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()

    def testGetEventsByDay(self):
        events = MultidayRecurringEventPage.events.byDay(dt.date(2020,1,1),
                                                         dt.date(2020,1,31))
        self.assertEqual(len(events), 31)
        evod = events[0]
        self.assertEqual(evod.date, dt.date(2020,1,1))
        self.assertEqual(len(evod.days_events), 0)
        self.assertEqual(len(evod.continuing_events), 1)
        thisEvent = evod.continuing_events[0]
        self.assertEqual(thisEvent.title, "Ice Festival")
        evod = events[10]
        self.assertEqual(evod.date, dt.date(2020,1,11))
        self.assertEqual(len(evod.continuing_events), 1)
        thisEvent = evod.continuing_events[0]
        self.assertEqual(thisEvent.title, "Ice Festival")

    def testStatus(self):
        with freeze_timetz("2019-12-20 13:00:00"):
            self.assertEqual(self.event.status_text, "")
        with freeze_timetz("2020-01-02 13:00:00"):
            self.assertEqual(self.event.status_text, "This event has started.")
        with freeze_timetz("2020-02-03 17:00:00"):
            self.assertEqual(self.event.status_text, "These events have finished.")

    def testAt(self):
        self.assertEqual(self.event.at.strip(), "")

    @freeze_timetz("2035-04-03 10:00:00")
    def testNextDate(self):
        self.assertEqual(self.event.next_date, None)

    @freeze_timetz("2035-04-03 10:00:00")
    def testPrevDate(self):
        self.assertEqual(self.event.prev_date, dt.date(2019, 12, 23))

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Joyous Widgets
# ------------------------------------------------------------------------------
import sys
import json
import datetime as dt
from django.templatetags.static import static
from django.forms import Media
from django.utils.formats import get_format
from django.utils import timezone
from django.utils.translation import gettext as _
from django.forms.widgets import MultiWidget, NumberInput, Select, \
        CheckboxSelectMultiple, FileInput
from django.template.loader import render_to_string
from wagtail.admin.widgets import AdminDateInput, AdminTimeInput
from dateutil.parser import parse as dt_parse
from .utils.recurrence import Weekday, Recurrence, DAILY, WEEKLY, MONTHLY, YEARLY
from .utils.manythings import toTheOrdinal
from .utils.names import WEEKDAY_NAMES, WEEKDAY_ABBRS, MONTH_ABBRS

# ------------------------------------------------------------------------------
class Time12hrInput(AdminTimeInput):
    """
    Display and Edit time fields in a 12hr format
    """
    template_name = 'joyous/widgets/time12hr_input.html'

    def __init__(self, attrs=None):
        super().__init__(attrs=attrs, format=None)

    def format_value(self, value):
        if isinstance(value, (dt.datetime, dt.time)):
            return value.strftime("%I:%M%P") # %P for lower case am/pm
        else:
            return value

    @property
    def media(self):
        return Media(js=[static('joyous/js/vendor/moment-2.22.0.min.js'),
                         static("joyous/js/time12hr_admin.js")])

# ------------------------------------------------------------------------------
(EVERY_DAY, SAME_DAY, DAY_OF_MONTH) = (100, 101, 200)

class RecurrenceWidget(MultiWidget):
    """
    Widget for entering the rule of a recurrence
    """
    template_name = 'joyous/widgets/recurrence_widget.html'

    def __init__(self, attrs=None):
        freqOptions = [(DAILY,   _("Daily")),
                       (WEEKLY,  _("Weekly")),
                       (MONTHLY, _("Monthly")),
                       (YEARLY,  _("Yearly"))]
        ordOptions1 = [(1,  toTheOrdinal(1)),
                       (2,  toTheOrdinal(2)),
                       (3,  toTheOrdinal(3)),
                       (4,  toTheOrdinal(4)),
                       (5,  toTheOrdinal(5)),
                       (-1, toTheOrdinal(-1)),
                       (EVERY_DAY, _("Every")),
                       (SAME_DAY, _("The Same"))]
        ordOptions2 = [(None, ""),
                       (1,    toTheOrdinal(1)),
                       (2,    toTheOrdinal(2)),
                       (3,    toTheOrdinal(3)),
                       (4,    toTheOrdinal(4)),
                       (5,    toTheOrdinal(5)),
                       (-1,   toTheOrdinal(-1))]
        dayOptions1  = enumerate(WEEKDAY_ABBRS)
        dayOptions2  = [(None, "")] + list(enumerate(WEEKDAY_NAMES))
        dayOptions3  = list(enumerate(WEEKDAY_NAMES)) +\
                       [(DAY_OF_MONTH, _("Day of the month"))]
        monthOptions = enumerate(MONTH_ABBRS[1:], 1)

        numAttrs = {'min': 1, 'max': 366}
        disableAttrs = {'disabled': True}
        if attrs:
            numAttrs.update(attrs)
            disableAttrs.update(attrs)
        widgets = [AdminDateInput(attrs=attrs),
                   Select(attrs=attrs, choices=freqOptions),         #1
                   NumberInput(attrs=numAttrs),
                   CheckboxSelectMultiple(attrs=attrs, choices=dayOptions1),
                   NumberInput(attrs=numAttrs),
                   AdminDateInput(attrs=attrs),                      #5
                   Select(attrs=attrs, choices=ordOptions1),
                   Select(attrs=attrs, choices=dayOptions3),
                   Select(attrs=disableAttrs, choices=ordOptions2),
                   Select(attrs=disableAttrs, choices=dayOptions2),
                   Select(attrs=disableAttrs, choices=ordOptions2),  #10
                   Select(attrs=disableAttrs, choices=dayOptions2),
                   CheckboxSelectMultiple(attrs=attrs, choices=monthOptions) ]
        super().__init__(widgets, attrs)

    def decompress(self, value):
        wdayChoices = []
        ordChoices  = [SAME_DAY,    None, None]
        dayChoices  = [DAY_OF_MONTH, None, None]
        monChoices  = []
        if isinstance(value, Recurrence):
            if value.freq == WEEKLY:
                if value.byweekday:
                    wdayChoices = [day.weekday for day in value.byweekday]
            elif value.freq in (MONTHLY, YEARLY):
                if value.byweekday:
                    if len(value.byweekday) == 7 and all(not day.n for day in value.byweekday):
                        ordChoices[0] = EVERY_DAY
                        dayChoices[0] = DAY_OF_MONTH
                    else:
                        for (i, day) in enumerate(value.byweekday[:3]):
                            dayChoices[i] = day.weekday
                            ordChoices[i] = day.n or EVERY_DAY
                elif value.bymonthday:
                    ordChoices[0] = value.bymonthday[0]
                    if value.dtstart.day == ordChoices[0]:
                        ordChoices[0] = SAME_DAY
                    dayChoices[0] = DAY_OF_MONTH
                if value.bymonth:
                    monChoices = value.bymonth
                else:
                    value.dtstart.month
            return [value.dtstart,
                    value.freq,      #1
                    value.interval,
                    wdayChoices,
                    value.count,
                    value.until,     #5
                    ordChoices[0],
                    dayChoices[0],
                    ordChoices[1],
                    dayChoices[1],
                    ordChoices[2],   #10
                    dayChoices[2],
                    monChoices]
        else:
            return [None,
                    None,            #1
                    1,
                    wdayChoices,
                    None,
                    None,            #5
                    ordChoices[0],
                    dayChoices[0],
                    ordChoices[1],
                    dayChoices[1],
                    ordChoices[2],   #10
                    dayChoices[2],
                    monChoices]

    def get_context(self, name, value, attrs):
        """
        Return the context to use with the widget's template, e.g.
        {'widget': {
            'attrs': {'id': "id_repeat", 'required': True},
            'is_hidden': False,
            'name': "repeat",
            'required': True,
            'subwidgets': [... the context of all the component subwigets...],
            'template_name': "joyous/widgets/recurrence_widget.html",
            'value': "Tuesdays",
            'value_s': "Tuesdays",
            'value_r': "DTSTART:20181201\nRRULE:FREQ=WEEKLY;WKST=SU;BYDAY=TU",
            }}}
        """
        context = super().get_context(name, value, attrs)
        context['widget']['value_s'] = str(value)
        context['widget']['value_r'] = repr(value)
        return context

    def value_from_datadict(self, data, files, name):
        values = [widget.value_from_datadict(data, files, "{}_{}".format(name, i))
                  for i, widget in enumerate(self.widgets)]
        try:
            def toIntOrNone(value):
                return int(value) if value else None
            dtstart     = dt_parse(values[0]) if values[0] else None
            freq        = toIntOrNone(values[1])
            interval    = toIntOrNone(values[2]) or None
            #count      = toIntOrNone(values[4]) or None
            dtuntil     = dt_parse(values[5]) if values[5] else None
            ordChoices  = [toIntOrNone(values[6]),
                           toIntOrNone(values[8]),
                           toIntOrNone(values[10])]
            dayChoices  = [toIntOrNone(values[7]),
                           toIntOrNone(values[9]),
                           toIntOrNone(values[11])]
            wdayChoices = None
            mdayChoices = None
            monChoices  = None
            if freq == WEEKLY:
                if values[3]:
                    wdayChoices = [int(day) for day in values[3]]
            elif freq in (MONTHLY, YEARLY):
                if dayChoices[0] == DAY_OF_MONTH:    # day of the month
                    if ordChoices[0] == EVERY_DAY:      # every day, == daily
                        wdayChoices = range(7)
                    elif ordChoices[0] == SAME_DAY:     # the same day of the month
                        mdayChoices = None
                    else:
                        mdayChoices = [ordChoices[0]]
                else:                         # a day of the week
                    if ordChoices[0] == EVERY_DAY:      # every of this weekday
                        wdayChoices = [Weekday(dayChoices[0])]
                    elif ordChoices[0] == SAME_DAY:     # the same weekday of the month
                        wdayNum = (dtstart.day - 1) // 7 + 1
                        wdayChoices = [Weekday(dayChoices[0], wdayNum)]
                    else:
                        wdayChoices = [Weekday(dayChoices[0], ordChoices[0])]
                if dayChoices[1] != None and ordChoices[1] != None:
                    wdayChoices.append(Weekday(dayChoices[1], ordChoices[1]))
                if dayChoices[2] != None and ordChoices[2] != None:
                    wdayChoices.append(Weekday(dayChoices[2], ordChoices[2]))
                if freq == YEARLY:
                    if values[12]:
                        monChoices = [int(month) for month in values[12]]

            retval = Recurrence(dtstart    = dtstart,
                                freq       = freq,
                                interval   = interval,
                                byweekday  = wdayChoices,
                                #count      = count,
                                until      = dtuntil,
                                bymonthday = mdayChoices,
                                bymonth    = monChoices)
        except (TypeError, ValueError):
            retval = None
        return retval

    @property
    def media(self):
        media = super()._get_media()
        media += Media(css={'all': [static("joyous/css/recurrence_admin.css")]},
                       js=[static("joyous/js/recurrence_admin.js")])
        return media

# ------------------------------------------------------------------------------
class ExceptionDateInput(AdminDateInput):
    """
    Display and Edit the dates which are the exceptions to a recurrence rule
    """
    template_name = 'joyous/widgets/exception_date_input.html'

    def __init__(self, attrs=None, format=None):
        super().__init__(attrs=attrs, format=format)
        self.overrides_repeat = None

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        config = {
            'dayOfWeekStart': get_format('FIRST_DAY_OF_WEEK'),
            'format':         self.js_format,
        }
        context['widget']['valid_dates'] = json.dumps(self.valid_dates())
        context['widget']['config_json'] = json.dumps(config)
        return context

    def valid_dates(self):
        valid_dates = None # null in JS
        if self.overrides_repeat:
            today = timezone.localdate()
            past = (today - dt.timedelta(days=200)).replace(day=1)
            future = (today + dt.timedelta(days=600)).replace(day=1)
            valid_dates = ["{:%Y%m%d}".format(occurence) for occurence in
                           self.overrides_repeat.between(past, future, inc=True)]
        return valid_dates

    @property
    def media(self):
        return Media(css={'all': [static("joyous/css/recurrence_admin.css")]},
                     js=[static("joyous/js/recurrence_admin.js")])

# TODO Should probably also do validation on the returned date?
# that would require ExceptionDateField and ExceptionDateFormField :(
# or else use custom form for page validation?
# https://github.com/torchbox/wagtail/pull/1867

# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# vtimezone functions copied from https://github.com/pimutils/khal
# ------------------------------------------------------------------------------
import datetime as dt
import pytz
from ls.joyous.formats.vtimezone import create_timezone
from django.test import TestCase


# ------------------------------------------------------------------------------
class TestVDt(TestCase):
    def setUp(self):
        self.berlin = pytz.timezone('Europe/Berlin')
        self.bogota = pytz.timezone('America/Bogota')

    def testBerlin(self):
        vberlin_std = b'\r\n'.join(
            [b'BEGIN:STANDARD',
             b'DTSTART;VALUE=DATE-TIME:20141026T020000',
             b'TZNAME:CET',
             b'TZOFFSETFROM:+0200',
             b'TZOFFSETTO:+0100',
             b'END:STANDARD',
             ])
        vberlin_dst = b'\r\n'.join(
            [b'BEGIN:DAYLIGHT',
             b'DTSTART;VALUE=DATE-TIME:20150329T030000',
             b'TZNAME:CEST',
             b'TZOFFSETFROM:+0100',
             b'TZOFFSETTO:+0200',
             b'END:DAYLIGHT',
             ])
        atime = dt.datetime(2014, 10, 28, 10, 10)
        vberlin = create_timezone(self.berlin, atime, atime).to_ical()
        self.assertIn(b'TZID:Europe/Berlin', vberlin)
        self.assertIn(vberlin_std, vberlin)
        self.assertIn(vberlin_dst, vberlin)

    def testBerlinRdate(self):
        vberlin_std = b'\r\n'.join(
            [b'BEGIN:STANDARD',
             b'DTSTART;VALUE=DATE-TIME:20141026T020000',
             b'RDATE:20151025T020000,20161030T020000',
             b'TZNAME:CET',
             b'TZOFFSETFROM:+0200',
             b'TZOFFSETTO:+0100',
             b'END:STANDARD',
             ])
        vberlin_dst = b'\r\n'.join(
            [b'BEGIN:DAYLIGHT',
             b'DTSTART;VALUE=DATE-TIME:20150329T030000',
             b'RDATE:20160327T030000',
             b'TZNAME:CEST',
             b'TZOFFSETFROM:+0100',
             b'TZOFFSETTO:+0200',
             b'END:DAYLIGHT',
             ])
        atime = dt.datetime(2014, 10, 28, 10, 10)
        btime = dt.datetime(2016, 10, 28, 10, 10)
        vberlin = create_timezone(self.berlin, atime, btime).to_ical()
        assert b'TZID:Europe/Berlin' in vberlin
        assert vberlin_std in vberlin
        assert vberlin_dst in vberlin

    def testBogota(self):
        vbogota = [b'BEGIN:VTIMEZONE',
                   b'TZID:America/Bogota',
                   b'BEGIN:STANDARD',
                   b'DTSTART;VALUE=DATE-TIME:19930403T230000',
                   b'TZNAME:COT',
                   b'TZOFFSETFROM:-0400',
                   b'TZOFFSETTO:-0500',
                   b'END:STANDARD',
                   b'END:VTIMEZONE',
                   b'']
        pytz_version = tuple(int(x) for x in pytz.__version__.split("."))
        if pytz_version > (2017, 1):
            vbogota[4] = b'TZNAME:-05'
            vbogota.insert(4, b'RDATE:20380118T221407')
        atime = dt.datetime(2014, 10, 28, 10, 10)
        assert create_timezone(self.bogota, atime, atime).to_ical().split(b'\r\n') == vbogota

    def testPST(self):
        vpacific = [b'BEGIN:VTIMEZONE',
                    b'TZID:Etc/GMT-8',
                    b'BEGIN:STANDARD',
                    b'DTSTART;VALUE=DATE-TIME:16010101T000000',
                    b'RDATE:16010101T000000',
                    b'TZNAME:Etc/GMT-8',
                    b'TZOFFSETFROM:+0800',
                    b'TZOFFSETTO:+0800',
                    b'END:STANDARD',
                    b'END:VTIMEZONE',
                    b'']
        pst = pytz.timezone('Etc/GMT-8')
        atime = dt.datetime(2014, 10, 28, 10, 10)
        btime = dt.datetime(2016, 10, 28, 10, 10)
        vtz = create_timezone(pst, atime, btime)
        assert vtz.to_ical().split(b'\r\n') == vpacific

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Test Edit Handlers
# ------------------------------------------------------------------------------
import sys
import datetime as dt
from unittest import skipIf, skipUnless
from django.test import RequestFactory, TestCase, override_settings
from django.conf import settings
from django.contrib.auth.models import User
from django.utils import timezone
from django.utils.formats import get_format
from wagtail.admin.edit_handlers import get_form_for_model
from wagtail.admin.widgets import AdminTimeInput, AdminDateInput
from wagtail.core.models import Site, Page
from ls.joyous.models.events import (CancellationPageForm, RecurringEventPageForm,
                                     HiddenNumDaysPanel)
from ls.joyous.models import (CalendarPage, CancellationPage, RecurringEventPage,
                              MultidayRecurringEventPage)
from ls.joyous.utils.recurrence import Recurrence, MONTHLY, YEARLY, TU, FR
from ls.joyous.edit_handlers import ExceptionDatePanel, ConcealedPanel
from ls.joyous.widgets import Time12hrInput, ExceptionDateInput
from .testutils import datetimetz, freeze_timetz, getPage
import ls.joyous.edit_handlers
import importlib
from wagtail import VERSION as _wt_version
WagtailVersion = _wt_version[:3]

# ------------------------------------------------------------------------------
class TestExceptionDatePanel(TestCase):
    def setUp(self):
        self.home = getPage("/home/")
        self.user = User.objects.create_superuser('i', 'i@joy.test', 's3(r3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()
        self.event = RecurringEventPage(slug      = "leaders-meeting",
                                        title     = "Leaders' Meeting",
                                        repeat    = Recurrence(dtstart=dt.date(2016,2,16),
                                                               freq=MONTHLY,
                                                               byweekday=[TU(3)]),
                                        time_from = dt.time(19),
                                        tz        = "Asia/Tokyo")
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()

    def _getRequest(self):
        request = RequestFactory().get("/test")
        request.user = self.user
        request.session = {}
        request.site = Site.objects.get(is_default_site=True)
        return request

    def testWidget(self):
        self.assertIs(ExceptionDatePanel.widget, ExceptionDateInput)

    @skipUnless(WagtailVersion < (2, 5, 0), "Wagtail >=2.5")
    def testBindWithoutOverrides23(self):
        cancellation = CancellationPage(owner = self.user,
                                        except_date = dt.date(2019,1,21))
        Form = get_form_for_model(CancellationPage, form_class=CancellationPageForm)
        form = Form(instance=cancellation, parent_page=self.event)
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to_model(CancellationPage)
        panel = panel.bind_to_instance(instance=cancellation,
                                       form=form,
                                       request=self._getRequest())
        self.assertIsNotNone(panel.form)
        self.assertIsNone(panel.instance.overrides)

    @skipUnless(WagtailVersion < (2, 5, 0), "Wagtail >=2.5")
    def testBindOverridesRepeat23(self):
        cancellation = CancellationPage(owner = self.user,
                                        overrides = self.event,
                                        except_date = dt.date(2019,1,21))
        Form = get_form_for_model(CancellationPage, form_class=CancellationPageForm)
        form = Form(instance=cancellation, parent_page=self.event)
        widget = form['except_date'].field.widget
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to_model(CancellationPage)
        panel = panel.bind_to_instance(instance=cancellation,
                                       form=form,
                                       request=self._getRequest())
        self.assertIs(widget.overrides_repeat, self.event.repeat)
        self.assertIsNone(panel.exceptionTZ)

    @skipUnless(WagtailVersion < (2, 5, 0), "Wagtail >=2.5")
    @timezone.override("America/Los_Angeles")
    def testBindExceptionTZ23(self):
        cancellation = CancellationPage(owner = self.user,
                                        overrides = self.event,
                                        except_date = dt.date(2019,1,21))
        Form = get_form_for_model(CancellationPage, form_class=CancellationPageForm)
        form = Form(instance=cancellation, parent_page=self.event)
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to_model(CancellationPage)
        panel = panel.bind_to_instance(instance=cancellation,
                                       form=form,
                                       request=self._getRequest())
        self.assertEqual(panel.exceptionTZ, "Asia/Tokyo")

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testBindWithoutForm25(self):
        cancellation = CancellationPage(owner = self.user,
                                        except_date = dt.date(2019,1,21))
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to(instance=cancellation)
        self.assertIsNone(panel.form)

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testBindWithoutOverrides25(self):
        cancellation = CancellationPage(owner = self.user,
                                        except_date = dt.date(2019,1,21))
        Form = get_form_for_model(CancellationPage, form_class=CancellationPageForm)
        form = Form(instance=cancellation, parent_page=self.event)
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to(instance=cancellation)
        panel = panel.bind_to(request=self._getRequest())
        panel = panel.bind_to(form=form)
        self.assertIsNotNone(panel.form)
        self.assertIsNone(panel.instance.overrides)

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testBindOverridesRepeat25(self):
        cancellation = CancellationPage(owner = self.user,
                                        overrides = self.event,
                                        except_date = dt.date(2019,1,21))
        Form = get_form_for_model(CancellationPage, form_class=CancellationPageForm)
        form = Form(instance=cancellation, parent_page=self.event)
        widget = form['except_date'].field.widget
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to(instance=cancellation)
        panel = panel.bind_to(request=self._getRequest())
        panel = panel.bind_to(form=form)
        self.assertIs(widget.overrides_repeat, self.event.repeat)
        self.assertIsNone(panel.exceptionTZ)

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    @timezone.override("America/Los_Angeles")
    def testBindExceptionTZ25(self):
        cancellation = CancellationPage(owner = self.user,
                                        overrides = self.event,
                                        except_date = dt.date(2019,1,21))
        Form = get_form_for_model(CancellationPage, form_class=CancellationPageForm)
        form = Form(instance=cancellation, parent_page=self.event)
        panel = ExceptionDatePanel('except_date', classname='full-width')
        panel = panel.bind_to(instance=cancellation)
        panel = panel.bind_to(request=self._getRequest())
        panel = panel.bind_to(form=form)
        self.assertEqual(panel.exceptionTZ, "Asia/Tokyo")

# ------------------------------------------------------------------------------
@override_settings(JOYOUS_TIME_INPUT=12)
class TestTime12hrPanel(TestCase):
    def testWidget(self):
        importlib.reload(ls.joyous.edit_handlers)
        from ls.joyous.edit_handlers import TimePanel
        self.assertIs(TimePanel.widget, Time12hrInput)

    def testDefaultTimeInput(self):
        importlib.reload(ls.joyous.edit_handlers)
        self.assertIn('%I:%M%p', settings.TIME_INPUT_FORMATS)
        self.assertIn('%I%p', settings.TIME_INPUT_FORMATS)

    @override_settings(LANGUAGE_CODE="en-nz")
    def testNZLocaleTimeInput(self):
        importlib.reload(ls.joyous.edit_handlers)
        format = get_format('TIME_INPUT_FORMATS')
        self.assertIn('%I:%M%p', format)
        self.assertIn('%I%p', format)


# ------------------------------------------------------------------------------
class TestConcealedPanel(TestCase):
    def setUp(self):
        self.home = getPage("/home/")
        self.user = User.objects.create_superuser('i', 'i@joy.test', 's3(r3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()
        self.event = RecurringEventPage(slug      = "leaders-meeting",
                                        title     = "Leaders' Meeting",
                                        repeat    = Recurrence(dtstart=dt.date(2016,2,16),
                                                               freq=MONTHLY,
                                                               byweekday=[TU(3)]),
                                        time_from = dt.time(19),
                                        tz        = "Asia/Tokyo")
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()

    def _getRequest(self):
        request = RequestFactory().get("/test")
        request.user = self.user
        request.session = {}
        request.site = Site.objects.get(is_default_site=True)
        return request

    def testInit(self):
        panel = ConcealedPanel([], "Test", help_text="Nothing")
        self.assertEqual(panel._heading, "Test")
        self.assertEqual(panel._help_text, "Nothing")
        self.assertEqual(panel.heading, "")
        self.assertEqual(panel.help_text, "")

    @skipUnless(WagtailVersion < (2, 5, 0), "Wagtail >=2.5")
    def testConcealed23(self):
        Form = get_form_for_model(RecurringEventPage,
                                  form_class=RecurringEventPageForm)
        form = Form(instance=self.event, parent_page=self.calendar)
        panel = ConcealedPanel([], "Test")
        panel = panel.bind_to_model(RecurringEventPage)
        panel = panel.bind_to_instance(instance=self.event,
                                       form=form,
                                       request=self._getRequest())
        content = panel.render()
        self.assertEqual(content, "")
        self.assertEqual(panel.heading, "")
        self.assertEqual(panel.help_text, "")

    @skipUnless(WagtailVersion < (2, 5, 0), "Wagtail >=2.5")
    def testShown23(self):
        class ShownPanel(ConcealedPanel):
            def _show(self):
                return True

        Form = get_form_for_model(RecurringEventPage,
                                  form_class=RecurringEventPageForm)
        form = Form(instance=self.event, parent_page=self.calendar)
        panel = ShownPanel([], "Test", help_text="Nothing")
        panel = panel.bind_to_model(RecurringEventPage)
        panel = panel.bind_to_instance(instance=self.event,
                                       form=form,
                                       request=self._getRequest())
        content = panel.render()
        self.assertHTMLEqual(content, """
<fieldset>
    <legend>Test</legend>
    <ul class="fields">
    </ul>
</fieldset>
""")
        self.assertEqual(panel.heading, "Test")
        self.assertEqual(panel.help_text, "Nothing")

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testConcealed25(self):
        panel = ConcealedPanel([], "Test")
        panel = panel.bind_to(instance=self.event)
        panel = panel.bind_to(request=self._getRequest())
        content = panel.render()
        self.assertEqual(content, "")
        self.assertEqual(panel.heading, "")
        self.assertEqual(panel.help_text, "")

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testShown25(self):
        class ShownPanel(ConcealedPanel):
            def _show(self):
                return True

        panel = ShownPanel([], "Test", help_text="Nothing")
        panel = panel.bind_to(instance=self.event)
        self.assertEqual(panel.heading, "")
        self.assertEqual(panel.help_text, "")
        panel = panel.bind_to(request=self._getRequest())
        content = panel.render()
        self.assertHTMLEqual(content, """
<fieldset>
    <legend>Test</legend>
    <ul class="fields">
    </ul>
</fieldset>
""")
        self.assertEqual(panel.heading, "Test")
        self.assertEqual(panel.help_text, "Nothing")

# ------------------------------------------------------------------------------
class TestHiddenNumDaysPanel(TestCase):
    FIELD_CONTENT = """
<fieldset>
  <legend>Number of days</legend>
  <ul class="fields">
    <li>
      <div class="field integer_field number_input ">
        <div class="field-content">
          <div class="input  ">
            <input type="number" name="num_days" value="1" required id="id_num_days">
            <span></span>
          </div>
        </div>
      </div>
    </li>
  </ul>
</fieldset>
"""

    def setUp(self):
        self.home = getPage("/home/")
        self.user = User.objects.create_superuser('i', 'i@joy.test', 's3(r3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()
        self.event = RecurringEventPage(slug      = "leaders-meeting",
                                        title     = "Leaders' Meeting",
                                        repeat    = Recurrence(dtstart=dt.date(2016,2,16),
                                                               freq=MONTHLY,
                                                               byweekday=[TU(3)]),
                                        time_from = dt.time(19),
                                        tz        = "Asia/Tokyo")
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()
        Form = get_form_for_model(RecurringEventPage,
                                  form_class=RecurringEventPageForm)
        self.form = Form(instance=self.event, parent_page=self.calendar)

    def _getRequest(self):
        request = RequestFactory().get("/test")
        request.user = self.user
        request.session = {}
        request.site = Site.objects.get(is_default_site=True)
        return request

    @skipUnless(WagtailVersion < (2, 5, 0), "Wagtail >=2.5")
    def testHidden24(self):
        panel = HiddenNumDaysPanel()
        panel = panel.bind_to_model(RecurringEventPage)
        panel = panel.bind_to_instance(instance=self.event,
                                       form=self.form,
                                       request=self._getRequest())
        content = panel.render_as_object()
        self.assertEqual(content, "")
        content = panel.render_as_field()
        self.assertEqual(content, "")

    @skipUnless(WagtailVersion < (2, 5, 0), "Wagtail >=2.5")
    def testShowWith2Days24(self):
        self.event.num_days = 2
        panel = HiddenNumDaysPanel()
        panel = panel.bind_to_model(RecurringEventPage)
        panel = panel.bind_to_instance(instance=self.event,
                                       form=self.form,
                                       request=self._getRequest())
        content = panel.render_as_object()
        self.assertHTMLEqual(content, self.FIELD_CONTENT)

    @skipUnless(WagtailVersion < (2, 5, 0), "Wagtail >=2.5")
    def testShowMulidayRecurringEvent24(self):
        event = MultidayRecurringEventPage(slug      = "leaders-retreat",
                                           title     = "Leaders' Retreet",
                                           repeat    = Recurrence(dtstart=dt.date(2016,2,16),
                                                                  freq=YEARLY,
                                                                  bymonth=3,
                                                                  byweekday=[FR(1)]),
                                           time_from = dt.time(19),
                                           num_days  = 3,
                                           tz        = "Asia/Tokyo")
        self.calendar.add_child(instance=event)
        event.save_revision().publish()
        panel = HiddenNumDaysPanel()
        panel = panel.bind_to_model(RecurringEventPage)
        panel = panel.bind_to_instance(instance=event,
                                       form=self.form,
                                       request=self._getRequest())
        content = panel.render_as_object()
        self.assertHTMLEqual(content, self.FIELD_CONTENT)

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testHidden25(self):
        panel = HiddenNumDaysPanel()
        panel = panel.bind_to(instance=self.event,
                              request=self._getRequest(),
                              form=self.form)
        content = panel.render_as_object()
        self.assertEqual(content, "")
        content = panel.render_as_field()
        self.assertEqual(content, "")

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testShowWith2Days25(self):
        self.event.num_days = 2
        panel = HiddenNumDaysPanel()
        panel = panel.bind_to(instance=self.event,
                              request=self._getRequest(),
                              form=self.form)
        content = panel.render_as_object()
        self.assertHTMLEqual(content, self.FIELD_CONTENT)

    @skipUnless(WagtailVersion >= (2, 5, 0), "Wagtail <2.5")
    def testShowMulidayRecurringEvent25(self):
        event = MultidayRecurringEventPage(slug      = "leaders-retreat",
                                           title     = "Leaders' Retreet",
                                           repeat    = Recurrence(dtstart=dt.date(2016,2,16),
                                                                  freq=YEARLY,
                                                                  bymonth=3,
                                                                  byweekday=[FR(1)]),
                                           time_from = dt.time(19),
                                           num_days  = 3,
                                           tz        = "Asia/Tokyo")
        self.calendar.add_child(instance=event)
        event.save_revision().publish()
        panel = HiddenNumDaysPanel()
        panel = panel.bind_to(instance=event,
                              request=self._getRequest(),
                              form=self.form)
        content = panel.render_as_object()
        self.assertHTMLEqual(content, self.FIELD_CONTENT)

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Test Event Pages
# ------------------------------------------------------------------------------
import sys
import datetime as dt
import pytz
from django.contrib.auth.models import User
from django.contrib.auth.models import Group, Permission
from django.utils import translation
from wagtail.tests.utils import WagtailPageTests
from wagtail.tests.utils.form_data import nested_form_data, rich_text
from wagtail.core.models import Page
from ls.joyous.models import (SimpleEventPage, MultidayEventPage,
        RecurringEventPage, MultidayRecurringEventPage, ExtraInfoPage,
        CancellationPage, PostponementPage, RescheduleMultidayEventPage,
        CalendarPage, SpecificCalendarPage, GeneralCalendarPage)
from ls.joyous.models.groups import get_group_model
GroupPage = get_group_model()
from ls.joyous.utils.recurrence import Recurrence, WEEKLY, MO, WE, FR
from .testutils import skipUnlessSetup

# ------------------------------------------------------------------------------
class PageClassTests(WagtailPageTests):
    """
    Tests on the class definitions of pages - no instances required
    """
    def testCanCreateCalendar(self):
        self.assertCanCreateAt(Page, CalendarPage)

    def testCanCreateSpecificCalendar(self):
        self.assertCanCreateAt(Page, SpecificCalendarPage)

    def testCanCreateGeneralCalendar(self):
        self.assertCanCreateAt(Page, GeneralCalendarPage)

    def testCanCreateGroup(self):
        self.assertCanCreateAt(Page, GroupPage)

    def testCanCreateSimpleEvent(self):
        self.assertCanCreateAt(CalendarPage, SimpleEventPage)
        self.assertCanCreateAt(SpecificCalendarPage, SimpleEventPage)
        self.assertCanCreateAt(GeneralCalendarPage, SimpleEventPage)
        self.assertCanCreateAt(GroupPage, SimpleEventPage)
        self.assertCanNotCreateAt(Page, SimpleEventPage)

    def testCanCreateMultidayEvent(self):
        self.assertCanCreateAt(CalendarPage, MultidayEventPage)
        self.assertCanCreateAt(SpecificCalendarPage, MultidayEventPage)
        self.assertCanCreateAt(GeneralCalendarPage, MultidayEventPage)
        self.assertCanCreateAt(GroupPage, MultidayEventPage)
        self.assertCanNotCreateAt(Page, MultidayEventPage)
        self.assertCanNotCreateAt(SimpleEventPage, MultidayEventPage)

    def testCanCreateRecurringEvent(self):
        self.assertCanCreateAt(CalendarPage, RecurringEventPage)
        self.assertCanCreateAt(SpecificCalendarPage, RecurringEventPage)
        self.assertCanCreateAt(GeneralCalendarPage, RecurringEventPage)
        self.assertCanCreateAt(GroupPage, RecurringEventPage)
        self.assertCanNotCreateAt(Page, RecurringEventPage)

    def testCanCreateMultidayRecurringEvent(self):
        self.assertCanCreateAt(CalendarPage, MultidayRecurringEventPage)
        self.assertCanCreateAt(SpecificCalendarPage, MultidayRecurringEventPage)
        self.assertCanCreateAt(GeneralCalendarPage, MultidayRecurringEventPage)
        self.assertCanCreateAt(GroupPage, MultidayRecurringEventPage)
        self.assertCanNotCreateAt(Page, MultidayRecurringEventPage)

    def testCanCreateExtraInfo(self):
        self.assertCanCreateAt(RecurringEventPage, ExtraInfoPage)
        self.assertCanCreateAt(MultidayRecurringEventPage, ExtraInfoPage)
        self.assertCanNotCreateAt(CalendarPage, ExtraInfoPage)
        self.assertCanNotCreateAt(SpecificCalendarPage, ExtraInfoPage)
        self.assertCanNotCreateAt(GeneralCalendarPage, ExtraInfoPage)
        self.assertCanNotCreateAt(Page, ExtraInfoPage)

    def testCanCreateCancellation(self):
        self.assertCanCreateAt(RecurringEventPage, CancellationPage)
        self.assertCanCreateAt(MultidayRecurringEventPage, CancellationPage)
        self.assertCanNotCreateAt(CalendarPage, CancellationPage)
        self.assertCanNotCreateAt(SpecificCalendarPage, ExtraInfoPage)
        self.assertCanNotCreateAt(GeneralCalendarPage, ExtraInfoPage)
        self.assertCanNotCreateAt(GroupPage, CancellationPage)

    def testCanCreatePostponement(self):
        self.assertCanCreateAt(RecurringEventPage, PostponementPage)
        self.assertCanCreateAt(MultidayRecurringEventPage, RescheduleMultidayEventPage)
        self.assertCanNotCreateAt(CalendarPage, PostponementPage)
        self.assertCanNotCreateAt(SpecificCalendarPage, ExtraInfoPage)
        self.assertCanNotCreateAt(GeneralCalendarPage, ExtraInfoPage)
        self.assertCanNotCreateAt(MultidayEventPage, PostponementPage)

    def testSimpleEventAllows(self):
        self.assertAllowedParentPageTypes(SimpleEventPage, {CalendarPage,
                                                            SpecificCalendarPage,
                                                            GeneralCalendarPage,
                                                            GroupPage})
        self.assertAllowedSubpageTypes(SimpleEventPage, {})

    def testMultidayEventAllows(self):
        self.assertAllowedParentPageTypes(MultidayEventPage, {CalendarPage,
                                                              SpecificCalendarPage,
                                                              GeneralCalendarPage,
                                                              GroupPage})
        self.assertAllowedSubpageTypes(MultidayEventPage, {})

    def testRecurringEventAllows(self):
        self.assertAllowedParentPageTypes(RecurringEventPage, {CalendarPage,
                                                               SpecificCalendarPage,
                                                               GeneralCalendarPage,
                                                               GroupPage})
        self.assertAllowedSubpageTypes(RecurringEventPage,
                                       {ExtraInfoPage, CancellationPage,
                                        PostponementPage})


    def testMultidayRecurringEventAllows(self):
        self.assertAllowedParentPageTypes(MultidayRecurringEventPage,
                                          {CalendarPage,
                                           SpecificCalendarPage,
                                           GeneralCalendarPage,
                                           GroupPage})
        self.assertAllowedSubpageTypes(MultidayRecurringEventPage,
                                       {ExtraInfoPage, CancellationPage,
                                        RescheduleMultidayEventPage})

    def testExtraInfoAllows(self):
        self.assertAllowedParentPageTypes(ExtraInfoPage,
                                          {RecurringEventPage,
                                           MultidayRecurringEventPage})
        self.assertAllowedSubpageTypes(ExtraInfoPage, {})

    def testCancellationAllows(self):
        self.assertAllowedParentPageTypes(CancellationPage,
                                          {RecurringEventPage,
                                           MultidayRecurringEventPage})
        self.assertAllowedSubpageTypes(CancellationPage, {})

    def testPostponementAllows(self):
        self.assertAllowedParentPageTypes(PostponementPage,
                                          {RecurringEventPage})
        self.assertAllowedSubpageTypes(PostponementPage, {})

    def testRescheduleMultidayEventAllows(self):
        self.assertAllowedParentPageTypes(RescheduleMultidayEventPage,
                                          {MultidayRecurringEventPage})
        self.assertAllowedSubpageTypes(RescheduleMultidayEventPage, {})

    def testCalendarVerboseName(self):
        self.assertEqual(CalendarPage.get_verbose_name(),
                         "Calendar page")

    def testSpecificCalendarVerboseName(self):
        self.assertEqual(SpecificCalendarPage.get_verbose_name(),
                         "Specific calendar page")

    def testGeneralCalendarVerboseName(self):
        self.assertEqual(GeneralCalendarPage.get_verbose_name(),
                         "General calendar page")

    def testSimpleEventVerboseName(self):
        self.assertEqual(SimpleEventPage.get_verbose_name(),
                         "Event page")

    def testMultidayEventVerboseName(self):
        self.assertEqual(MultidayEventPage.get_verbose_name(),
                         "Multiday event page")

    def testRecurringEventVerboseName(self):
        self.assertEqual(RecurringEventPage.get_verbose_name(),
                         "Recurring event page")

    def testMultidayRecurringEventVerboseName(self):
        self.assertEqual(MultidayRecurringEventPage.get_verbose_name(),
                         "Multiday recurring event page")

    def testExtraInfoVerboseName(self):
        self.assertEqual(ExtraInfoPage.get_verbose_name(),
                         "Extra event information")

    def testCancellationVerboseName(self):
        self.assertEqual(CancellationPage.get_verbose_name(),
                         "Cancellation")

    def testPostponementVerboseName(self):
        self.assertEqual(PostponementPage.get_verbose_name(),
                         "Postponement")

    def testRescheduleMultidayEventVerboseName(self):
        self.assertEqual(RescheduleMultidayEventPage.get_verbose_name(),
                         "Postponement")

# ------------------------------------------------------------------------------
class PageClassTestsFrançais(WagtailPageTests):
    def setUp(self):
        translation.activate('fr')

    def tearDown(self):
        translation.deactivate()

    def testCalendarVerboseName(self):
        self.assertEqual(CalendarPage.get_verbose_name(),
                         "Page de calendrier")

    def testSpecificCalendarVerboseName(self):
        self.assertEqual(SpecificCalendarPage.get_verbose_name(),
                         "Page de calendrier spécifique")

    def testGeneralCalendarVerboseName(self):
        self.assertEqual(GeneralCalendarPage.get_verbose_name(),
                         "Page de calendrier générale")

    def testSimpleEventVerboseName(self):
        self.assertEqual(SimpleEventPage.get_verbose_name(),
                         "Page de l'événement")

    def testMultidayEventVerboseName(self):
        self.assertEqual(MultidayEventPage.get_verbose_name(),
                         "Page de l'événement sur plusieurs jours")

    def testRecurringEventVerboseName(self):
        self.assertEqual(RecurringEventPage.get_verbose_name(),
                         "Page d'événement récurrent")

    def testMultidayRecurringEventVerboseName(self):
        self.assertEqual(MultidayRecurringEventPage.get_verbose_name(),
                         "Page d'événements récurrents sur plusieurs jours")

    def testExtraInfoVerboseName(self):
        self.assertEqual(ExtraInfoPage.get_verbose_name(),
                         "Informations supplémentaires sur l'événement")

    def testCancellationVerboseName(self):
        self.assertEqual(CancellationPage.get_verbose_name(),
                         "Annulation")

    def testPostponementVerboseName(self):
        self.assertEqual(PostponementPage.get_verbose_name(),
                         "Report")

    def testRescheduleMultidayEventVerboseName(self):
        self.assertEqual(RescheduleMultidayEventPage.get_verbose_name(),
                         "Report")

# ------------------------------------------------------------------------------
class PageInstanceTests(WagtailPageTests):
    """
    Tests with instantiated pages
    """
    def setUp(self):
        self.home = Page.objects.get(slug='home')
        self.user = User.objects.create_user('i', 'i@joy.test', 's3(r3t')
        self.user.groups.add(Group.objects.get(name="Moderators"))
        self.client.force_login(self.user)
        try:
            self.home = Page.objects.get(slug='home')
            self.group = GroupPage(slug  = "test-group",
                                   title = "Test Group")
            self.home.add_child(instance=self.group)
            self.event = RecurringEventPage(slug      = "test-meeting",
                                            title     = "Test Meeting",
                                            repeat    = Recurrence(dtstart=dt.date(2009,8,7),
                                                                   freq=WEEKLY,
                                                                   byweekday=[MO,WE,FR]),
                                            time_from = dt.time(13))
            self.group.add_child(instance=self.event)
        except:
            pass

    def testCanCreateCalendar(self):
        self.assertCanCreate(self.home, CalendarPage,
                             nested_form_data({'title': "Calendar",
                                               'intro': rich_text("<h4>What's happening</h4>"),
                                               'default_view': "M"}))

    def testCanCreateSpecificCalendar(self):
        SpecificCalendarPage.is_creatable = True
        self.assertCanCreate(self.home, SpecificCalendarPage,
                             nested_form_data({'title': "Calendar",
                                               'intro': rich_text("<h4>What's happening</h4>"),
                                               'default_view': "M"}))

    def testCanCreateGeneralCalendar(self):
        GeneralCalendarPage.is_creatable = True
        self.assertCanCreate(self.home, GeneralCalendarPage,
                             nested_form_data({'title': "Calendar",
                                               'intro': rich_text("<h4>What's happening</h4>"),
                                               'default_view': "L"}))

    def testCanCreateGroup(self):
        self.assertCanCreate(self.home, GroupPage,
                             nested_form_data({'title': "Moreporks Club",
                                               'intro': rich_text("<h4>Welcome to the club</h4>")}))

    @skipUnlessSetup("group")
    def testCanCreateSimpleEvent(self):
        self.assertCanCreate(self.group, SimpleEventPage,
                             nested_form_data({'title':      "Mouse Hunt",
                                               'date':       dt.date(1987,6,5),
                                               'tz':         pytz.timezone("Pacific/Auckland"),
                                               'details':    rich_text("<p>Hello Micee</p>")}))

    @skipUnlessSetup("group")
    def testCanCreateMultidayEvent(self):
        self.assertCanCreate(self.group, MultidayEventPage,
                             nested_form_data({'title':      "Camp QA",
                                               'date_from':  dt.date(1987,7,10),
                                               'date_to':    dt.date(1987,7,12),
                                               'time_from':  dt.time(17),
                                               'time_to':    dt.time(14,30),
                                               'tz':         pytz.timezone("Pacific/Auckland"),
                                               'details':    rich_text("<p>Hello World</p>")}))

    @skipUnlessSetup("group")
    def testCanCreateRecurringEvent(self):
        self.assertCanCreate(self.group, RecurringEventPage,
                             nested_form_data({'title':      "Stand up",
                                               'repeat_0':   dt.date(1987,6,7),
                                               'repeat_1':   2,                    # weekly
                                               'repeat_2':   1,                    # every week
                                               'repeat_3':   {0:0, 1:1, 2:2, 4:4}, # Mon,Tue,Wed,Fri
                                               'time_from':  dt.time(9),
                                               'time_to':    dt.time(10),
                                               'tz':         pytz.timezone("Pacific/Auckland"),
                                               'details':
                                                   rich_text("<p>Stand up straight!</p>")}))

    @skipUnlessSetup("group")
    def testCanCreateMultidayRecurringEvent(self):
        self.assertCanCreate(self.group, MultidayRecurringEventPage,
                             nested_form_data({'title':      "Team Retreat",
                                               'repeat_0':   dt.date(1987,8,7),
                                               'repeat_1':   0,                    # yearly
                                               'repeat_2':   1,                    # every year
                                               'repeat_6':   1,                    # the first
                                               'repeat_7':   4,                    # Friday of
                                               'repeat_12':  {8:8},                # August
                                               'num_days':   3,
                                               'time_from':  dt.time(17),
                                               'tz':         pytz.timezone("Pacific/Auckland"),
                                               'details':
                                                   rich_text("<p>Three days of T-E-A-M</p>")}))

    @skipUnlessSetup("event")
    def testCanCreateExtraInfo(self):
        self.assertCanCreate(self.event, ExtraInfoPage,
                             nested_form_data({'overrides':  self.event.id,
                                               'except_date':dt.date(2009,8,14),
                                               'extra_information':
                                                   rich_text("<h3>A special announcement</h3>")}))

    @skipUnlessSetup("event")
    def testCanCreateCancellation(self):
        self.assertCanCreate(self.event, CancellationPage,
                             nested_form_data({'overrides':            self.event.id,
                                               'except_date':          dt.date(2009,8,14),
                                               'cancellation_title':   "Meeting Cancelled",
                                               'cancellation_details':
                                                   rich_text("<p>No meeting today</p>")}))

    @skipUnlessSetup("event")
    def testCanCreatePostponement(self):
        self.assertCanCreate(self.event, PostponementPage,
                             nested_form_data({'overrides':            self.event.id,
                                               'except_date':          dt.date(2009,8,16),
                                               'cancellation_title':   "Meeting Postponed",
                                               'cancellation_details':
                                                   rich_text("<p>Meeting will be held tommorrow</p>"),
                                               'postponement_title':   "Test Meeting",
                                               'date':                 dt.date(2009,8,15),
                                               'time_from':            dt.time(13)}))

    def testCanCancelMultidayEvent(self):
        event2 = MultidayRecurringEventPage(slug      = "test-session",
                                            title     = "Test Session",
                                            repeat    = Recurrence(dtstart=dt.date(2009,8,7),
                                                                   freq=WEEKLY,
                                                                   byweekday=[MO,WE,FR]),
                                            num_days  = 2,
                                            time_from = dt.time(10))
        self.group.add_child(instance=event2)
        self.assertCanCreate(event2, CancellationPage,
                             nested_form_data({'overrides':            self.event.id,
                                               'except_date':          dt.date(2009,8,14),
                                               'cancellation_title':   "Session Cancelled" }))

    def testCanRescheduleMultidayEvent(self):
        event2 = MultidayRecurringEventPage(slug      = "test-session",
                                            title     = "Test Session",
                                            repeat    = Recurrence(dtstart=dt.date(2009,8,7),
                                                                   freq=WEEKLY,
                                                                   byweekday=[MO,WE,FR]),
                                            num_days  = 2,
                                            time_from = dt.time(10))
        self.group.add_child(instance=event2)
        self.assertCanCreate(event2, RescheduleMultidayEventPage,
                             nested_form_data({'overrides':            self.event.id,
                                               'except_date':          dt.date(2009,8,16),
                                               'postponement_title':   "Shortened cycle",
                                               'date':                 dt.date(2009,8,16),
                                               'num_days':             1,
                                               'time_from':            dt.time(10)}))

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Google Calendar Page Export Handler
# ------------------------------------------------------------------------------
import datetime as dt
from collections import OrderedDict
from urllib.parse import urlencode
import pytz
from icalendar import vPeriod
from django.http import HttpResponseRedirect
from ..models import (SimpleEventPage, MultidayEventPage, RecurringEventPage,
        EventExceptionBase)
from ..utils.telltime import getAwareDatetime

# ------------------------------------------------------------------------------
class GoogleCalendarHandler:
    """Redirect to a new Google Calendar event"""
    def serve(self, page, request, *args, **kwargs):
        gevent = self._makeFromPage(page)
        if gevent:
            return HttpResponseRedirect(gevent.url)

    def _makeFromPage(self, page):
        if isinstance(page, SimpleEventPage):
            return SimpleGEvent.fromPage(page)
        elif isinstance(page, MultidayEventPage):
            return MultidayGEvent.fromPage(page)
        elif isinstance(page, RecurringEventPage):
            return RecurringGEvent.fromPage(page)
        elif isinstance(page, EventExceptionBase):
            return RecurringGEvent.fromPage(page.overrides)

# ------------------------------------------------------------------------------
class GEvent(OrderedDict):
    def set(self, name, value):
        if value:
            self[name] = value

    @classmethod
    def fromPage(cls, page):
        gevent = cls()
        gevent.set('action',   "TEMPLATE")
        gevent.set('text',     page.title)
        gevent.set('details',  page.details)
        gevent.set('location', page.location)
        return gevent

    @property
    def url(self):
        retval  = "http://www.google.com/calendar/event?"
        retval += urlencode(self)
        return retval

# ------------------------------------------------------------------------------
class SimpleGEvent(GEvent):
    @classmethod
    def fromPage(cls, page):
        gevent = super().fromPage(page)
        # FIXME support Anniversary date type events?
        dtstart = getAwareDatetime(page.date, page.time_from, page.tz, dt.time.min)
        dtend   = getAwareDatetime(page.date, page.time_to, page.tz, dt.time.max)
        gevent.set('dates', vPeriod((dtstart, dtend)).to_ical().decode())
        if page.tz != pytz.utc:
            gevent.set('ctz', page.tz.zone)
        return gevent

# ------------------------------------------------------------------------------
class MultidayGEvent(GEvent):
    @classmethod
    def fromPage(cls, page):
        gevent = super().fromPage(page)
        # FIXME support Anniversary date type events?
        dtstart = getAwareDatetime(page.date_from, page.time_from, page.tz, dt.time.min)
        dtend   = getAwareDatetime(page.date_to, page.time_to, page.tz, dt.time.max)
        gevent.set('dates', vPeriod((dtstart, dtend)).to_ical().decode())
        if page.tz != pytz.utc:
            gevent.set('ctz', page.tz.zone)
        return gevent

# ------------------------------------------------------------------------------
class RecurringGEvent(GEvent):
    @classmethod
    def fromPage(cls, page):
        gevent = super().fromPage(page)
        minDt   = pytz.utc.localize(dt.datetime.min)
        # FIXME support Anniversary date type events?
        dtstart = page._getMyFirstDatetimeFrom() or minDt
        dtend   = page._getMyFirstDatetimeTo()   or minDt
        gevent.set('dates', vPeriod((dtstart, dtend)).to_ical().decode())
        if page.tz != pytz.utc:
            gevent.set('ctz', page.tz.zone)
        gevent.set('recur', "RRULE:" + page.repeat._getRrule())
        # TODO: try and confirm...
        # Google doesn't accept EXDATE or RDATE here :-(
        return gevent

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Test Recurring Event Page
# ------------------------------------------------------------------------------
import sys
import datetime as dt
import pytz
import calendar
from django.test import TestCase, RequestFactory
from django.contrib.auth.models import User
from django.utils import timezone
from wagtail.core.models import Site, Page
from ls.joyous.utils.recurrence import Recurrence
from ls.joyous.utils.recurrence import DAILY, WEEKLY, MONTHLY
from ls.joyous.utils.recurrence import TU, TH, SA, SU, EVERYWEEKDAY
from ls.joyous.models.calendar import CalendarPage
from ls.joyous.models.events import (RecurringEventPage, CancellationPage,
                                     PostponementPage)
from .testutils import datetimetz, freeze_timetz

# ------------------------------------------------------------------------------
class Test(TestCase):
    def setUp(self):
        Site.objects.update(hostname="joy.test")
        self.user = User.objects.create_user('i', 'i@joy.test', 's3cr3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        Page.objects.get(slug='home').add_child(instance=self.calendar)
        self.calendar.save()
        self.calendar.save_revision().publish()
        self.event = RecurringEventPage(owner = self.user,
                                        slug  = "lug-meetup",
                                        title = "Linux Users Group Meetup",
                                        repeat = Recurrence(dtstart=dt.date(2017,8,5),
                                                            freq=MONTHLY,
                                                            byweekday=[TU(1)]),
                                        time_from = dt.time(18,30),
                                        time_to   = dt.time(20),
                                        location  = "6 Mackay St, Greymouth (upstairs)")
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()

    def testGetEventsByDay(self):
        events = RecurringEventPage.events.byDay(dt.date(2017,8,1),
                                                 dt.date(2017,10,31))
        self.assertEqual(len(events), 92)
        evod = events[35]
        self.assertEqual(evod.date, dt.date(2017,9,5))
        self.assertEqual(len(evod.all_events), 1)
        self.assertEqual(len(evod.days_events), 1)
        self.assertEqual(len(evod.continuing_events), 0)

    def testStatus(self):
        pastEvent = RecurringEventPage(owner = self.user,
                                       slug  = "past",
                                       title = "Past Event",
                                       repeat = Recurrence(dtstart=dt.date(2008,2,1),
                                                           until=dt.date(2008,5,4),
                                                           freq=WEEKLY,
                                                           byweekday=[SA,SU]))
        self.calendar.add_child(instance=pastEvent)
        self.assertEqual(pastEvent.status, "finished")
        self.assertEqual(pastEvent.status_text, "These events have finished.")
        now = timezone.localtime()
        earlier = now - dt.timedelta(hours=1)
        if earlier.date() != now.date():
            earlier = datetimetz(now.date(), dt.time.min)
        nowEvent = RecurringEventPage(owner = self.user,
                                      slug  = "now",
                                      title = "Now Event",
                                      repeat = Recurrence(dtstart=dt.date(2010,1,1),
                                                          freq=DAILY),
                                      time_from = earlier.time(),
                                      time_to   = dt.time.max)
        self.calendar.add_child(instance=nowEvent)
        self.assertEqual(nowEvent.status, "started")
        self.assertEqual(nowEvent.status_text, "This event has started.")
        today = timezone.localdate()
        notToday = [weekday for weekday in EVERYWEEKDAY if weekday.weekday != today.weekday()]
        pastAndFutureEvent = RecurringEventPage(owner = self.user,
                                                slug  = "not-today",
                                                title = "Any day but today",
                                                repeat = Recurrence(dtstart=dt.date(2009,8,7),
                                                                    freq=WEEKLY,
                                                                    byweekday=notToday))
        self.calendar.add_child(instance=pastAndFutureEvent)
        self.assertIsNone(pastAndFutureEvent.status)
        self.assertEqual(pastAndFutureEvent.status_text, "")

    @freeze_timetz("2008-05-04 09:01")
    def testJustFinishedStatus(self):
        event = RecurringEventPage(owner = self.user,
                                   slug  = "breakfast1",
                                   title = "Breakfast-in-bed",
                                   repeat = Recurrence(dtstart=dt.date(2008,2,1),
                                                       until=dt.date(2008,5,9),
                                                       freq=WEEKLY,
                                                       byweekday=[SA,SU]),
                                      time_from = dt.time(8),
                                      time_to   = dt.time(9))
        self.calendar.add_child(instance=event)
        self.assertEqual(event.status, "finished")

    @freeze_timetz("2008-05-04 07:00")
    def testLastOccurenceCancelledStatus(self):
        event = RecurringEventPage(owner = self.user,
                                   slug  = "breakfast2",
                                   title = "Breakfast-in-bed",
                                   repeat = Recurrence(dtstart=dt.date(2008,2,1),
                                                       until=dt.date(2008,5,9),
                                                       freq=WEEKLY,
                                                       byweekday=[SA,SU]),
                                   time_from = dt.time(8),
                                   time_to   = dt.time(9))
        self.calendar.add_child(instance=event)
        cancellation = CancellationPage(owner = self.user,
                                        overrides = event,
                                        except_date = dt.date(2008, 5, 4),
                                        cancellation_title   = "Fire in the kitchen",
                                        cancellation_details = "The bacon fat is burning")
        event.add_child(instance=cancellation)
        self.assertEqual(event.status, "finished")

    @freeze_timetz("2008-05-04 12:00")
    def testPostponementOccurenceLast(self):
        request = RequestFactory().get("/test")
        request.user = self.user
        request.session = {}
        event = RecurringEventPage(owner = self.user,
                                   slug  = "breakfast3",
                                   title = "Breakfast-in-bed",
                                   repeat = Recurrence(dtstart=dt.date(2008,2,1),
                                                       until=dt.date(2008,5,9),
                                                       freq=WEEKLY,
                                                       byweekday=[SA,SU]),
                                   time_from = dt.time(8),
                                   time_to   = dt.time(9))
        self.calendar.add_child(instance=event)
        postponement = PostponementPage(owner = self.user,
                                        overrides = event,
                                        except_date = dt.date(2008, 5, 3),
                                        postponement_title = "Breakfast in Bed owed from May",
                                        date      = dt.date(2008, 5, 24),
                                        time_from = dt.time(8),
                                        time_to   = dt.time(9))
        event.add_child(instance=postponement)
        self.assertIsNone(event.status)
        self.assertEqual(event._nextOn(request),
                         '<a class="inline-link" href="/events/breakfast3/2008-05-03-postponement/">Saturday 24th of May at 8am</a>')

    def testWhen(self):
        self.assertEqual(self.event.when, "The first Tuesday of the month at 6:30pm to 8pm")

    def testAt(self):
        self.assertEqual(self.event.at.strip(), "6:30pm")

    def testCurrentDt(self):
        lugDt = self.event._current_datetime_from
        self.assertEqual(lugDt.time(), dt.time(18,30))
        self.assertEqual(lugDt.date().weekday(), 1)
        self.assertLess(lugDt.date().day, 8)
        movieNight = RecurringEventPage(owner = self.user,
                                        slug  = "movies",
                                        title = "Movies",
                                        repeat = Recurrence(dtstart=dt.date(2005,2,1),
                                                            freq=WEEKLY,
                                                            byweekday=[TH,]),
                                        time_from = dt.time(20,15),
                                        time_to   = dt.time(21,30))
        self.calendar.add_child(instance=movieNight)
        now = timezone.localtime()
        myday = now.date()
        startTime = dt.time(20,15)
        if now.time() > startTime:
            myday += dt.timedelta(days=1)
        thursday = myday + dt.timedelta(days=(3-myday.weekday())%7)
        self.assertEqual(movieNight._current_datetime_from,
                         datetimetz(thursday, startTime))

    def testFutureDt(self):
        lugDt = self.event._future_datetime_from
        self.assertEqual(lugDt.time(), dt.time(18,30))
        self.assertEqual(lugDt.date().weekday(), 1)
        self.assertLess(lugDt.date().day, 8)
        movieNight = RecurringEventPage(owner = self.user,
                                        slug  = "movies",
                                        title = "Movies",
                                        repeat = Recurrence(dtstart=dt.date(2005,2,1),
                                                            freq=WEEKLY,
                                                            byweekday=[TH,]),
                                        time_from = dt.time(20,15),
                                        time_to   = dt.time(21,30))
        self.calendar.add_child(instance=movieNight)
        now = timezone.localtime()
        myday = now.date()
        startTime = dt.time(20,15)
        if now.time() > startTime:
            myday += dt.timedelta(days=1)
        thursday = myday + dt.timedelta(days=(3-myday.weekday())%7)
        self.assertEqual(movieNight._future_datetime_from,
                         datetimetz(thursday, startTime))

    def testPastDt(self):
        lugDt = self.event._past_datetime_from
        self.assertEqual(lugDt.time(), dt.time(18,30))
        self.assertEqual(lugDt.date().weekday(), 1)
        self.assertLess(lugDt.date().day, 8)
        movieNight = RecurringEventPage(owner = self.user,
                                        slug  = "movies",
                                        title = "Movies",
                                        repeat = Recurrence(dtstart=dt.date(2005,2,1),
                                                            freq=WEEKLY,
                                                            byweekday=[TH,]),
                                        time_from = dt.time(20,15),
                                        time_to   = dt.time(21,30))
        self.calendar.add_child(instance=movieNight)
        now = timezone.localtime()
        myday = now.date()
        startTime = dt.time(20,15)
        if now.time() < startTime:
            myday -= dt.timedelta(days=1)
        thursday = myday - dt.timedelta(days=(myday.weekday()-3)%7)
        self.assertEqual(movieNight._past_datetime_from,
                         datetimetz(thursday, startTime))

    def testGroup(self):
        self.assertIsNone(self.event.group)

    def testOccursOn(self):
        self.assertIs(self.event._occursOn(dt.date(2018,3,6)), True)
        self.assertIs(self.event._occursOn(dt.date(2018,3,13)), False)

# ------------------------------------------------------------------------------
class TestTZ(TestCase):
    def setUp(self):
        self.home = Page.objects.get(slug='home')
        self.user = User.objects.create_user('i', 'i@ok.test', 's3cr3t')
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()
        self.event = RecurringEventPage(owner = self.user,
                                        slug  = "code-for-boston",
                                        title = "Code for Boston",
                                        repeat    = Recurrence(dtstart=dt.date(2017,1,1),
                                                               freq=WEEKLY,
                                                               byweekday=[TU]),
                                        time_from = dt.time(19),
                                        time_to   = dt.time(21,30),
                                        tz = pytz.timezone("US/Eastern"),
                                        location  = "4th Floor, 1 Broadway, Cambridge, MA")
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()

    def testGetEventsByLocalDay(self):
        events = RecurringEventPage.events.byDay(dt.date(2018,4,1),
                                                 dt.date(2018,4,30))
        self.assertEqual(len(events), 30)
        evod1 = events[3]
        self.assertEqual(evod1.date, dt.date(2018,4,4))
        self.assertEqual(len(evod1.days_events), 1)
        self.assertEqual(len(evod1.continuing_events), 0)

    @freeze_timetz("2017-05-31")
    def testLocalWhen(self):
        with timezone.override("America/Los_Angeles"):
            self.assertEqual(self.event.when,
                             "Tuesdays at 4pm to 6:30pm")
        with timezone.override("Australia/Perth"):
            self.assertEqual(self.event.when,
                             "Wednesdays at 7am to 9:30am")

    @timezone.override("America/Los_Angeles")
    def testLocalAt(self):
        self.assertEqual(self.event.at, "4pm")

    @timezone.override("America/Los_Angeles")
    def testCurrentLocalDt(self):
        when = self.event._current_datetime_from
        self.assertEqual(when.tzinfo.zone, "America/Los_Angeles")
        self.assertEqual(when.weekday(), calendar.TUESDAY)

    @timezone.override("America/Los_Angeles")
    def testFutureLocalDt(self):
        when = self.event._future_datetime_from
        self.assertEqual(when.tzinfo.zone, "America/Los_Angeles")
        self.assertEqual(when.weekday(), calendar.TUESDAY)

    @timezone.override("Pacific/Auckland")
    def testPastLocalDt(self):
        when = self.event._past_datetime_from
        self.assertEqual(when.tzinfo.zone, "Pacific/Auckland")
        self.assertEqual(when.weekday(), calendar.WEDNESDAY)

    @timezone.override("Pacific/Kiritimati")
    def testExtremeTimeZones(self):
        lions = RecurringEventPage(owner = self.user,
                                   slug  = "pago-pago-lions",
                                   title = "Pago Pago Lions Club",
                                   repeat = Recurrence(dtstart=dt.date(2015,2,1),
                                                       freq=MONTHLY,
                                                       byweekday=[TH(1),TH(3)]),
                                   time_from = dt.time(23,0),
                                   tz = pytz.timezone("Pacific/Pago_Pago"),
                                   location = "Lions Den, Tafuna, PagoPago",
                                   website = "http://www.lionsclubs.org.nz")
        self.calendar.add_child(instance=lions)
        self.assertEqual(lions.when,
                         "The Saturday after the first Thursday and "
                         "Saturday after the third Thursday of the month at 12am")

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Test Hooks
# ------------------------------------------------------------------------------
import sys
from unittest.mock import Mock, patch
from django.test import TestCase, RequestFactory
from django.contrib.auth.models import User
from wagtail.core.models import Page
from ls.joyous.models.calendar import CalendarPage, CalendarPageForm
from ls.joyous.wagtail_hooks import handlePageExport, stashRequest

# ------------------------------------------------------------------------------
class TestWagtailHooks(TestCase):
    def setUp(self):
        self.home = Page.objects.get(slug='home')
        self.user = User.objects.create_user('i', 'i@j.test', 's3(r3t')
        self.page = CalendarPage(owner  = self.user,
                                 slug  = "events",
                                 title = "Events")
        self.home.add_child(instance=self.page)
        self.page.save_revision().publish()

    @patch("ls.joyous.wagtail_hooks.ICalHandler")
    def testICalExport(self, Handler):
        request = RequestFactory().get("/test?format=ical")
        handlePageExport(self.page, request, [], {})
        Handler.assert_called_with()
        Handler().serve.assert_called_with(self.page, request, [], {})

    @patch("ls.joyous.wagtail_hooks.GoogleCalendarHandler")
    def testGoogleExport(self, Handler):
        request = RequestFactory().get("/test?format=google")
        handlePageExport(self.page, request, [], {})
        Handler.assert_called_with()
        Handler().serve.assert_called_with(self.page, request, [], {})

    @patch("ls.joyous.wagtail_hooks.RssHandler")
    def testRssExport(self, Handler):
        request = RequestFactory().get("/test?format=rss")
        handlePageExport(self.page, request, [], {})
        Handler.assert_called_with()
        Handler().serve.assert_called_with(self.page, request, [], {})

    @patch("ls.joyous.wagtail_hooks.NullHandler")
    def testNullExport(self, Handler):
        request = RequestFactory().get("/test")
        handlePageExport(self.page, request, [], {})
        Handler.assert_called_with()
        Handler().serve.assert_called_with(self.page, request, [], {})

    def testStashRequest(self):
        request = RequestFactory().get("/test")
        stashRequest(request, self.page)
        self.assertEqual(getattr(self.page, '__joyous_edit_request'), request)
        delattr(self.page, '__joyous_edit_request')

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
import sys
import datetime as dt
from dateutil.rrule import rrule
from django.test import TestCase
from ls.joyous.utils.recurrence import Recurrence, Weekday
from ls.joyous.utils.recurrence import MO, TU, WE, TH, FR, SA, SU
from ls.joyous.utils.recurrence import YEARLY, MONTHLY, WEEKLY, DAILY
from .testutils import datetimetz

# ------------------------------------------------------------------------------
class TestWeekday(TestCase):
    def testStr(self):
        self.assertEqual(str(Weekday(0)), "Monday")
        self.assertEqual(str(Weekday(4,1)), "first Friday")
        self.assertEqual(str(Weekday(4,-1)), "last Friday")
        self.assertEqual(str(SA), "Saturday")
        self.assertEqual(str(FR(3)), "third Friday")

    def testGetWhen(self):
        self.assertEqual(Weekday(0)._getWhen(0), "Monday")
        self.assertEqual(FR(1)._getWhen(0), "first Friday")
        self.assertEqual(SU._getWhen(1), "Monday")
        self.assertEqual(WE._getWhen(-2), "Monday")
        self.assertEqual(FR(1)._getWhen(-1), "Thursday before the first Friday")
        self.assertEqual(SU(1)._getWhen(2), "Tuesday after the first Sunday")

    def testRepr(self):
        self.assertEqual(repr(Weekday(0)), "MO")
        self.assertEqual(repr(Weekday(4,2)), "+2FR")
        self.assertEqual(repr(SA), "SA")
        self.assertEqual(repr(FR(3)), "+3FR")
        self.assertEqual(repr(WE(-2)), "-2WE")

# ------------------------------------------------------------------------------
class TestRecurrence(TestCase):
    def testInitStr(self):
        with self.assertRaises(ValueError):
            Recurrence("DTSTART:19970902T090000\n"
                       "RRULE:FREQ=DAILY;INTERVAL=3\n"
                       "RRULE:FREQ=DAILY;INTERVAL=4")

    def testInitRecurrence(self):
        rr1 = Recurrence(dtstart=dt.date(2009, 1, 1),
                         freq=WEEKLY,
                         byweekday=[MO,TU,WE,TH,FR])
        rr2 = Recurrence(rr1)
        self.assertEqual(rr2.freq, WEEKLY)

    def testInitRrule(self):
        rr1 = rrule(dtstart=dt.date(2009, 1, 1),
                    freq=WEEKLY,
                    byweekday=[MO,TU,WE,TH,FR])
        rr2 = Recurrence(rr1)
        self.assertEqual(rr2.freq, WEEKLY)

    def testEq(self):
        rr1 = rrule(dtstart=dt.datetime(2009, 1, 1, 0, 0, 1),
                    freq=WEEKLY,
                    byweekday=[MO,TU,WE,TH,FR])
        rr2 = Recurrence(dtstart=dt.date(2009, 1, 1),
                         freq=WEEKLY,
                         byweekday=[MO,TU,WE,TH,FR])
        rr3 = Recurrence("DTSTART:20090101\n"
                         "RRULE:FREQ=WEEKLY;WKST=SU;BYDAY=MO,TU,WE,TH,FR")
        rr4 = rrule(dtstart=dt.date(2009, 1, 1),
                    freq=WEEKLY,
                    byweekday=[MO,TU,WE,TH,FR],
                    until=dt.date(2009, 1, 10))
        self.assertEqual(Recurrence(rr1), rr2)
        self.assertEqual(rr2, rr1)
        self.assertEqual(rr1, rr2)
        self.assertEqual(rr2, rr2)
        self.assertEqual(rr2, rr3)
        self.assertNotEqual(rr2, 99)
        self.assertNotEqual(rr2, rr4)

    def testRepr(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=WEEKLY,
                        count=9,
                        byweekday=[MO,TU,WE,TH,FR])
        self.assertEqual(repr(rr),
                        "DTSTART:20090101\n"
                        "RRULE:FREQ=WEEKLY;WKST=SU;COUNT=9;BYDAY=MO,TU,WE,TH,FR")
        self.assertEqual(rr.count, rr.getCount())
        rr = Recurrence(dtstart=dt.date(2011, 1, 1),
                        freq=DAILY,
                        interval=2,
                        until=dt.date(2011,1,11))
        self.assertEqual(repr(rr),
                        "DTSTART:20110101\n"
                        "RRULE:FREQ=DAILY;INTERVAL=2;WKST=SU;UNTIL=20110111")
        rr = Recurrence(dtstart=dt.date(2012, 1, 1),
                        freq=YEARLY,
                        bymonth=[1,2],
                        byweekday=range(7),
                        until=dt.date(2012,1,31))
        self.assertEqual(repr(rr),
                        "DTSTART:20120101\n"
                        "RRULE:FREQ=YEARLY;WKST=SU;UNTIL=20120131;"
                        "BYDAY=MO,TU,WE,TH,FR,SA,SU;BYMONTH=1,2")
        rr = Recurrence(dtstart=dt.date(2015, 10, 1),
                        freq=MONTHLY,
                        bymonth=range(1,12),
                        byweekday=[(SU(-1))])
        self.assertEqual(repr(rr),
                        "DTSTART:20151001\n"
                        "RRULE:FREQ=MONTHLY;WKST=SU;BYDAY=-1SU;BYMONTH=1,2,3,4,5,6,7,8,9,10,11")

    def testParse(self):
        rr = Recurrence("DTSTART:20090101\n"
                        "RRULE:FREQ=WEEKLY;WKST=SU;BYDAY=MO,TU,WE,TH,FR;COUNT=9")
        self.assertEqual(rr.dtstart, dt.date(2009, 1, 1))
        self.assertEqual(rr.count, 9)
        self.assertCountEqual(rr.byweekday, [MO,TU,WE,TH,FR])

    def testParseNoDtstart(self):
        rr = Recurrence("RRULE:FREQ=DAILY;WKST=SU")
        self.assertEqual(rr.freq, DAILY)

    def testRoundtrip(self):
        rrStr = "DTSTART:20151001\n" \
                "RRULE:FREQ=MONTHLY;WKST=SU;BYDAY=-1SU;BYMONTH=1,2,3,4,5,6,7,8,9,10,11"
        self.assertEqual(repr(Recurrence(rrStr)), rrStr)
        rrStr = "DTSTART:20141001\n" \
                "RRULE:FREQ=MONTHLY;WKST=SU;UNTIL=20141001;BYMONTHDAY=1,-1"   # first&last
        self.assertEqual(repr(Recurrence(rrStr)), rrStr)

    def testFrequency(self):
        rr = Recurrence(freq=10)
        self.assertEqual(rr.frequency, "unsupported_frequency_10")

    def testGetRrule(self):
        rr = Recurrence(dtstart=dt.date(2011, 1, 1),
                        freq=DAILY,
                        interval=2,
                        until=dt.date(2011,1,11))
        self.assertEqual(rr._getRrule(),
                        "FREQ=DAILY;INTERVAL=2;WKST=SU;UNTIL=20110111")
        with self.assertRaises(TypeError):
            rr._getRrule(untilDt=dt.datetime(2011,1,11))
        self.assertEqual(rr._getRrule(untilDt=dt.datetime(2011,1,11,23,59,59,
                                                          tzinfo=dt.timezone.utc)),
                        "FREQ=DAILY;INTERVAL=2;WKST=SU;UNTIL=20110111T235959Z")

# ------------------------------------------------------------------------------
class TestGetWhen(TestCase):
    def testDaily(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1), freq=DAILY)
        self.assertEqual(rr._getWhen(2), "Daily")

    def testEvery2Days(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        interval=2,
                        freq=DAILY)
        self.assertEqual(rr._getWhen(3), "Every 2 days")

    def testMonEveryFortnight(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        interval=2,
                        freq=WEEKLY,
                        byweekday=MO)
        self.assertEqual(rr._getWhen(0), "Fortnightly on Mondays")

    def testMonEvery6Weeks(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        interval=6,
                        freq=WEEKLY,
                        byweekday=MO)
        self.assertEqual(rr._getWhen(0), "Every 6 weeks on Mondays")

    def testEveryday(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=MONTHLY,
                        byweekday=[MO,TU,WE,TH,FR,SA,SU])
        self.assertEqual(rr._getWhen(0), "Everyday")

    def testFirstMonMonthly(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=MONTHLY,
                        byweekday=MO(1))
        self.assertEqual(rr._getWhen(0), "The first Monday of the month")

    def testMonEvery2Months(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        interval=2,
                        freq=MONTHLY,
                        byweekday=MO)
        self.assertEqual(rr._getWhen(0), "Every Monday, every 2 months")

    def testLastSatSeptEvery2Years(self):
        rr = Recurrence(dtstart=dt.date(2018, 9, 29),
                        interval=2,
                        freq=YEARLY,
                        byweekday=SA(-1),
                        bymonth=9)
        self.assertEqual(rr._getWhen(0, numDays=5), "The last Saturday of September, every 2 years for 5 days")

    def test1st(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=MONTHLY,
                        bymonthday=1)
        self.assertEqual(rr._getWhen(0), "The first day of the month")

    def test22ndOffsetNeg1(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=YEARLY,
                        bymonthday=22,
                        bymonth=5)
        self.assertEqual(rr._getWhen(-1), "The 21st day of May")

    def test30thOffset1(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=MONTHLY,
                        bymonthday=30)
        self.assertEqual(rr._getWhen(1), "The day after the 30th day of the month")

    def testMonWedFriOffset1(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=WEEKLY,
                        count=9,
                        byweekday=[MO,WE,FR])
        self.assertEqual(rr._getWhen(1), "Tuesdays, Thursdays and Saturdays")

    def test2ndAnd4thFriOffsetNeg1(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=MONTHLY,
                        byweekday=[FR(2),FR(4)])
        self.assertEqual(rr._getWhen(-1), "The Thursday before the second Friday and "
                                          "Thursday before the fourth Friday of the month")

    def test1stOffsetNeg1(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=MONTHLY,
                        bymonthday=1,
                        until=dt.date(2010,5,1))
        self.assertEqual(rr._getWhen(-1), "The last day of the month (until 30 April 2010)")

    def test3rdOffset2(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=MONTHLY,
                        bymonthday=3)
        self.assertEqual(rr._getWhen(2), "The fifth day of the month")

    def test1stJanAprMayOffsetNeg1(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=YEARLY,
                        bymonthday=1,
                        bymonth=[1,4,5])
        self.assertEqual(rr._getWhen(-1), "The last day of December, March and April")

    def testLastJulAugSepDecOffset1(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=YEARLY,
                        bymonthday=-1,
                        bymonth=[7,8,9,12])
        self.assertEqual(rr._getWhen(1),
                         "The first day of August, September, October and January")

    def test1stAndLast(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=MONTHLY,
                        bymonthday=[1,-1])
        self.assertEqual(rr._getWhen(0), "The first and the last day of the month")

    def test1stAndLastOffsetNeg1(self):
        rr = Recurrence(dtstart=dt.date(2009, 1, 1),
                        freq=MONTHLY,
                        bymonthday=[1,-1])
        self.assertEqual(rr._getWhen(-1), "The day before the first and the last day of the month")

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# ------------------------------------------------------------------------------
# Test Signals
# ------------------------------------------------------------------------------
import sys
import datetime as dt
from wagtail.core.models import Page
from django.test import TestCase, RequestFactory
from django.contrib.auth.models import User, AnonymousUser
from ls.joyous.models import RecurringEventPage, PostponementPage, GeneralCalendarPage
from ls.joyous.utils.recurrence import Recurrence, WEEKLY, MO, WE, FR
from ls.joyous.signals import identifyExpectantParent
from .testutils import freeze_timetz

# ------------------------------------------------------------------------------
class Test(TestCase):
    def setUp(self):
        self.home = Page.objects.get(slug='home')
        self.user = User.objects.create_user('j', 'j@joy.test', 's3(r3t')
        calendar = GeneralCalendarPage(owner = self.user,
                                       slug  = "events",
                                       title = "Events")
        self.home.add_child(instance=calendar)
        calendar.save_revision().publish()
        self.event = RecurringEventPage(slug   = "test-meeting",
                                        title  = "Test Meeting",
                                        repeat = Recurrence(dtstart=dt.date(1990,1,1),
                                                            freq=WEEKLY,
                                                            byweekday=[MO,WE,FR]),
                                        time_from = dt.time(13,30),
                                        time_to   = dt.time(16))
        calendar.add_child(instance=self.event)
        self.event.save_revision().publish()

    @freeze_timetz("1991-01-01 15:00")
    def testIdentifyExpectantParent(self):
        postponement = PostponementPage()
        identifyExpectantParent(self.testIdentifyExpectantParent,
                                parent=self.event, page=postponement)
        self.assertEqual(postponement.overrides, self.event)
        self.assertEqual(postponement.except_date, dt.date(1991,1,2))
        self.assertEqual(postponement.date, dt.date(1991,1,3))
        self.assertEqual(postponement.postponement_title, "Test Meeting")
        self.assertIsNone(postponement.category)
        self.assertEqual(postponement.num_days, 1)
        self.assertEqual(postponement.time_from, dt.time(13,30))
        self.assertEqual(postponement.time_to, dt.time(16))

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# Generated by Django 2.0.3 on 2018-05-21 23:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('joyous', '0004_auto_20180425_2355'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='cancellationpage',
            options={'default_manager_name': 'objects', 'verbose_name': 'Cancellation'},
        ),
        migrations.AlterModelOptions(
            name='extrainfopage',
            options={'default_manager_name': 'objects', 'verbose_name': 'Extra Event Information'},
        ),
        migrations.AlterModelOptions(
            name='multidayeventpage',
            options={'default_manager_name': 'objects', 'verbose_name': 'Multiday Event Page'},
        ),
        migrations.AlterModelOptions(
            name='postponementpage',
            options={'default_manager_name': 'objects', 'verbose_name': 'Postponement'},
        ),
        migrations.AlterModelOptions(
            name='recurringeventpage',
            options={'default_manager_name': 'objects', 'verbose_name': 'Recurring Event Page'},
        ),
        migrations.AlterModelOptions(
            name='simpleeventpage',
            options={'default_manager_name': 'objects', 'verbose_name': 'Event Page'},
        ),
        migrations.AlterField(
            model_name='multidayeventpage',
            name='group_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='joyous.GroupPage'),
        ),
        migrations.AlterField(
            model_name='postponementpage',
            name='group_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='joyous.GroupPage'),
        ),
        migrations.AlterField(
            model_name='recurringeventpage',
            name='group_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='joyous.GroupPage'),
        ),
        migrations.AlterField(
            model_name='simpleeventpage',
            name='group_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='joyous.GroupPage'),
        ),
    ]



# ------------------------------------------------------------------------------
# Test RSS Format
# ------------------------------------------------------------------------------
import sys
import os.path
import datetime as dt
import pytz
from io import BytesIO
from icalendar import vDatetime
from django.contrib.auth.models import User
from django.contrib.messages.storage.fallback import FallbackStorage
from django.test import TestCase, RequestFactory
from bs4 import BeautifulSoup
from lxml import etree
from django_bs_test import TestCase
from django.utils import timezone
from wagtail.core.models import Site, Page
from wagtail.images.models import Image
from wagtail.images.tests.utils import get_test_image_file
from ls.joyous.models.calendar import CalendarPage
from ls.joyous.models import (EventCategory, SimpleEventPage, MultidayEventPage,
        RecurringEventPage, PostponementPage, ExtraInfoPage)
from ls.joyous.models.groups import get_group_model
from ls.joyous.models.events import ThisEvent
from ls.joyous.utils.recurrence import Recurrence
from ls.joyous.utils.recurrence import WEEKLY, MONTHLY, TU, SA
from ls.joyous.formats.rss import RssHandler, EventEntry
from ls.joyous.formats.errors import CalendarTypeError
from freezegun import freeze_time
from .testutils import datetimetz
GroupPage = get_group_model()

# ------------------------------------------------------------------------------
class TestFeed(TestCase):
    @freeze_time("2016-03-24")
    def setUp(self):
        imgFile = get_test_image_file()
        imgFile.name = "logo.png"
        self.img = Image.objects.create(title="Logo", file=imgFile)
        imgName = os.path.splitext(os.path.basename(self.img.file.name))[0]
        self.rendName = "{}.width-350.format-png.png".format(imgName)
        Site.objects.update(hostname="joy.test")
        self.home = Page.objects.get(slug='home')
        self.user = User.objects.create_user('i', 'i@joy.test', 's3(R3t')
        self.requestFactory = RequestFactory()
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()
        self.event = RecurringEventPage(owner = self.user,
                                        slug  = "workshop",
                                        title = "Workshop",
                                        image = self.img,
                                        repeat    = Recurrence(dtstart=dt.date(2017,1,1),
                                                               freq=MONTHLY,
                                                               byweekday=[TU(1)],
                                                               until=dt.date(2017,12,26)))
        self.calendar.add_child(instance=self.event)
        self.event.save_revision().publish()
        self.handler = RssHandler()

    def tearDown(self):
        self.img.file.delete(False)
        for rend in self.img.renditions.all():
            rend.file.delete(False)

    def _getRequest(self, path="/"):
        request = self.requestFactory.get(path)
        request.user = self.user
        request.site = self.home.get_site()
        request.session = {}
        request._messages = FallbackStorage(request)
        request.POST = request.POST.copy()
        request.POST['action-publish'] = "action-publish"
        return request

    @freeze_time("2016-12-02")
    def testServe(self):
        response = self.handler.serve(self.calendar,
                                      self._getRequest("/events/"))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.get('Content-Type'), "application/xml; charset=utf-8")
        self.assertIn(b'<?xml version=\'1.0\' encoding=\'UTF-8\'?>', response.content)
        soup = BeautifulSoup(response.content, "xml")
        channel = soup.channel
        self.assertEqual(channel.title.string, "Events")
        self.assertEqual(channel.description.string, "Events")
        self.assertEqual(channel.link.string, "http://joy.test/events/")
        self.assertEqual(channel.generator.string, "ls.joyous")
        self.assertEqual(len(channel("image")), 1)
        image = channel.image
        self.assertEqual(image.url.string, "http://joy.test/static/joyous/img/logo.png")
        self.assertEqual(image.title.string, "Events")
        self.assertEqual(image.link.string, "http://joy.test/events/")
        self.assertEqual(channel.lastBuildDate.string, "Fri, 02 Dec 2016 00:00:00 +0000")
        self.assertEqual(len(channel("item")), 1)
        item = channel.item
        self.assertEqual(item.title.string, "Workshop")
        self.assertEqual(item.link.string, "http://joy.test/events/workshop/")
        self.assertEqual(item.enclosure.decode(),
                         '<enclosure length="773" type="image/png" '
                         'url="http://joy.test/media/images/{}"/>'.format(self.rendName))
        self.assertEqual(item.description.decode(), """<description>\n\n\n
  &lt;div class="joy-ev-when joy-field"&gt;
    The first Tuesday of the month (until 26 December 2017)
  &lt;/div&gt;\n
  &lt;div class="joy-ev-next-on joy-field"&gt;
    Next on Tuesday 3rd of January 2017 
  &lt;/div&gt;\n\n\n\n
&lt;div class="rich-text"&gt;&lt;/div&gt;\n</description>""")
        self.assertEqual(item.guid.get("isPermaLink"), "true")
        self.assertEqual(item.guid.string, "http://joy.test/events/workshop/")
        self.assertEqual(item.pubDate.string, "Thu, 24 Mar 2016 00:00:00 +0000")

    def testServeUnsupported(self):
        response = self.handler.serve(self.event,
                                      self._getRequest("/events/workshop"))
        self.assertIsNone(response)

    @freeze_time("2016-03-25")
    def testServeExtraInfo(self):
        info = ExtraInfoPage(owner = self.user,
                             overrides = self.event,
                             except_date = dt.date(2017,2,7),
                             extra_title = "System Demo",
                             extra_information = "<h3>System Demo</h3>")
        self.event.add_child(instance=info)
        info.save_revision().publish()
        response = self.handler.serve(self.calendar,
                                      self._getRequest("/events/"))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.get('Content-Type'), "application/xml; charset=utf-8")
        soup = BeautifulSoup(response.content, "xml")
        channel = soup.channel
        self.assertEqual(channel.title.string, "Events")
        self.assertEqual(len(channel("item")), 2)
        item = channel("item")[1]
        self.assertEqual(item.title.string, "System Demo")
        self.assertEqual(item.link.string, "http://joy.test/events/workshop/2017-02-07-extra-info/")
        self.assertEqual(item.enclosure.decode(),
                         '<enclosure length="773" type="image/png" '
                         'url="http://joy.test/media/images/{}"/>'.format(self.rendName))
        self.assertEqual(item.description.decode(), """<description>\n\n\n
  &lt;div class="joy-ev-when joy-field"&gt;
    Tuesday 7th of February 2017
  &lt;/div&gt;\n\n\n\n
&lt;div class="rich-text"&gt;&lt;h3&gt;System Demo&lt;/h3&gt;&lt;/div&gt;
&lt;div class="rich-text"&gt;&lt;/div&gt;\n</description>""")
        self.assertEqual(item.guid.get("isPermaLink"), "true")
        self.assertEqual(item.guid.string, "http://joy.test/events/workshop/2017-02-07-extra-info/")
        self.assertEqual(item.pubDate.string, "Fri, 25 Mar 2016 00:00:00 +0000")

    @freeze_time("2016-03-26")
    def testServePostponement(self):
        postponement = PostponementPage(owner = self.user,
                                        overrides = self.event,
                                        except_date = dt.date(2017,4,4),
                                        image = self.img,
                                        cancellation_title   = "Workshop Postponed",
                                        cancellation_details = "Workshop will take place next week",
                                        postponement_title   = "Workshop",
                                        date      = dt.date(2017, 4, 11),
                                        details   = "Interesting stuff")
        self.event.add_child(instance=postponement)
        postponement.save_revision().publish()
        response = self.handler.serve(self.calendar,
                                      self._getRequest("/events/"))
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content, "xml")
        channel = soup.channel
        self.assertEqual(channel.title.string, "Events")
        self.assertEqual(len(channel("item")), 2)
        item = channel("item")[1]
        self.assertEqual(item.title.string, "Workshop")
        self.assertEqual(item.link.string, "http://joy.test/events/workshop/2017-04-04-postponement/")
        self.assertEqual(item.enclosure.decode(),
                         '<enclosure length="773" type="image/png" '
                         'url="http://joy.test/media/images/{}"/>'.format(self.rendName))
        self.assertEqual(item.description.decode(), """<description>\n\n\n
  &lt;div class="joy-ev-when joy-field"&gt;
    Tuesday 11th of April 2017
  &lt;/div&gt;\n
  &lt;div class="joy-ev-from-when joy-field"&gt;
    Postponed from Tuesday 4th of April 2017
  &lt;/div&gt;\n\n\n\n
&lt;div class="rich-text"&gt;Interesting stuff&lt;/div&gt;\n</description>""")
        self.assertEqual(item.guid.get("isPermaLink"), "true")
        self.assertEqual(item.guid.string, "http://joy.test/events/workshop/2017-04-04-postponement/")
        self.assertEqual(item.pubDate.string, "Sat, 26 Mar 2016 00:00:00 +0000")

# ------------------------------------------------------------------------------
class TestEntry(TestCase):
    def setUp(self):
        imgFile = get_test_image_file()
        imgFile.name = "people.png"
        self.img = Image.objects.create(title="People", file=imgFile)
        imgName = os.path.splitext(os.path.basename(self.img.file.name))[0]
        self.rendName = "{}.width-350.format-png.png".format(imgName)
        Site.objects.update(hostname="joy.test")
        self.home = Page.objects.get(slug='home')
        self.user = User.objects.create_user('i', 'i@joy.test', 's3(R3t')
        self.requestFactory = RequestFactory()
        self.calendar = CalendarPage(owner = self.user,
                                     slug  = "events",
                                     title = "Events")
        self.home.add_child(instance=self.calendar)
        self.calendar.save_revision().publish()

    def tearDown(self):
        self.img.file.delete(False)
        for rend in self.img.renditions.all():
            rend.file.delete(False)

    def _getRequest(self, path="/"):
        request = self.requestFactory.get(path)
        request.user = self.user
        request.site = self.home.get_site()
        request.session = {}
        request._messages = FallbackStorage(request)
        request.POST = request.POST.copy()
        request.POST['action-publish'] = "action-publish"
        return request

    def testSetCategory(self):
        cat = EventCategory.objects.create(code="A1", name="AlphaOne")
        page = MultidayEventPage(owner = self.user,
                                 slug  = "road-trip",
                                 title = "Road Trip",
                                 date_from = dt.date(2016,11,1),
                                 date_to   = dt.date(2016,11,10),
                                 time_from = dt.time(10),
                                 category  = cat)
        self.calendar.add_child(instance=page)
        page.save_revision().publish()
        request = self._getRequest()
        thisEvent = ThisEvent(page.title, page, page.get_url(request))
        entry = EventEntry.fromEvent(thisEvent, request)
        self.assertEqual(entry.category(), [{'term': 'AlphaOne'}])
        rss = entry.rss_entry()
        self.assertEqual(etree.tostring(rss),
b"""<item><title>Road Trip</title><link>http://joy.test/events/road-trip/</link><description>\n\n\n
  &lt;div class="joy-ev-when joy-field"&gt;
    Tuesday 1st of November 2016 at 10am to Thursday 10th of November 2016\n  &lt;/div&gt;\n\n\n\n
&lt;div class="rich-text"&gt;&lt;/div&gt;
</description><guid isPermaLink="true">http://joy.test/events/road-trip/</guid><category>AlphaOne</category></item>""")

    def testSetImage(self):
        page = SimpleEventPage(owner = self.user,
                               slug  = "meetup",
                               title = "Meet Up",
                               image = self.img,
                               date  = dt.date(2016,10,21),
                               time_from = dt.time(16))
        self.calendar.add_child(instance=page)
        page.save_revision().publish()
        request = self._getRequest()
        thisEvent = ThisEvent(page.title, page, page.get_url(request))
        entry = EventEntry.fromEvent(thisEvent, request)
        self.assertEqual(entry.enclosure(), {
            'length': '773',
            'url': 'http://joy.test/media/images/{}'.format(self.rendName),
            'type': 'image/png'})

    def testSetDescription(self):
        group = GroupPage(slug = "sandmen", title = "Sandmen")
        self.home.add_child(instance=group)
        group.save_revision().publish()
        page = MultidayEventPage(owner = self.user,
                                 slug  = "road-trip",
                                 title = "Road Trip",
                                 date_from = dt.date(2016,11,1),
                                 date_to   = dt.date(2016,11,10),
                                 group_page = group)
        self.calendar.add_child(instance=page)
        page.save_revision().publish()
        request = self._getRequest()
        thisEvent = ThisEvent(page.title, page, page.get_url(request))
        entry = EventEntry.fromEvent(thisEvent, request)
        rss = entry.rss_entry()
        self.assertEqual(etree.tostring(rss),
b"""<item><title>Road Trip</title><link>http://joy.test/events/road-trip/</link><description>\n
  &lt;div class="joy-ev-who joy-field"&gt;
    &lt;a class="joy-ev-who__link" href="http://joy.test/sandmen/"&gt;Sandmen&lt;/a&gt;
  &lt;/div&gt;\n\n\n
  &lt;div class="joy-ev-when joy-field"&gt;
    Tuesday 1st of November 2016 to Thursday 10th of November 2016\n  &lt;/div&gt;\n\n\n\n
&lt;div class="rich-text"&gt;&lt;/div&gt;
</description><guid isPermaLink="true">http://joy.test/events/road-trip/</guid></item>""")

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------



# Generated by Django 2.0.7 on 2018-07-15 05:56

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('joyous', '0006_generalcalendarpage_specificcalendarpage'),
    ]

    operations = [
        migrations.AddField(
            model_name='multidayeventpage',
            name='uid',
            field=models.CharField(db_index=True, default=uuid.uuid4, editable=False, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='postponementpage',
            name='uid',
            field=models.CharField(db_index=True, default=uuid.uuid4, editable=False, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='recurringeventpage',
            name='uid',
            field=models.CharField(db_index=True, default=uuid.uuid4, editable=False, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='simpleeventpage',
            name='uid',
            field=models.CharField(db_index=True, default=uuid.uuid4, editable=False, max_length=255, null=True),
        ),
    ]
